////////////////////////////////////////////////////////////////////////////////
// Filename  : debug.c
// File Desc : Integrated Debugger
// Project   : NSFE (Nintendo Sixty Four Emulator)
// Release   : 0.2a (Developer Release)
// Copyright : NSFE is the Copyright (c) 1999, Brian Faber.  
//
// This software is a Windows implementation of TrueReality, Copyright (C) 1999,
// Niki W. Waibel, and is published under the terms of the GNU General Public 
// License, see below:
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and or modify it under
// the terms of the GNU General Public Licence as published by the Free Software
// Foundation; either version 2 of the Licence, or any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public Licence for more details
//
// You should have received a copy of the GNU General Public Licence along with 
// this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
// Ave, Cambridge, MA 02139, USA.
//
// NSFE Author Information:
//    Brian Faber, Chino, Califorina - USA
//    bevis590@gte.net
//
//    Ben Marsh, Townsville, Queensland - Australia
//    photi@ozemail.com.au
//
// Original TrueReality Author Information:
//    Niki W. Waibel, Reichenau 20, 6890 Lustenau, Austria - EUROPE
//    niki.waibel@xms.net
// -----------------------------------------------------------------------------
// Update History
//
// 00001 16-02-1998 "Initial Code Release.                                     "

#ifndef _DEBUG_H_
#include "debug.h"
#endif // _DEBUG_H


////////////////////////////////////////////////////////////////////////////////
// Define the Property Sheet Pages for the Register Debug Window

   int RegisterDebugWindow( void )
   {
      PROPSHEETPAGE   psp[7];
      PROPSHEETHEADER psh;

      // Define the General Purpose Register Debug Page

      psp[0].dwSize      = sizeof( PROPSHEETPAGE );
      psp[0].dwFlags     = PSP_USETITLE;
      psp[0].hInstance   = hInst;
      psp[0].pszTemplate = MAKEINTRESOURCE( IDD_GPREGS );
      psp[0].pszIcon     = NULL;
      psp[0].pfnDlgProc  = (DLGPROC)DebugGPRegs;
      psp[0].pszTitle    = "General Purpose";
      psp[0].lParam      = 0;

      // Define the CP0 Register Debug Page

      psp[1].dwSize      = sizeof( PROPSHEETPAGE );
      psp[1].dwFlags     = PSP_USETITLE;
      psp[1].hInstance   = hInst;
      psp[1].pszTemplate = MAKEINTRESOURCE( IDD_CP0REGS );
      psp[1].pszIcon     = NULL;
      psp[1].pfnDlgProc  = (DLGPROC)CP0Regs;
      psp[1].pszTitle    = "System Control";
      psp[1].lParam      = 0;

      // Define the CP1 Register Debug Page

      psp[2].dwSize      = sizeof( PROPSHEETPAGE );
      psp[2].dwFlags     = PSP_USETITLE;
      psp[2].hInstance   = hInst;
      psp[2].pszTemplate = MAKEINTRESOURCE( IDD_CP1REGS );
      psp[2].pszIcon     = NULL;
      psp[2].pfnDlgProc  = (DLGPROC)CP1Regs;
      psp[2].pszTitle    = "Floating Point";
      psp[2].lParam      = 0;

      // Define the Core CP2 Register Debug Page

      psp[3].dwSize      = sizeof( PROPSHEETPAGE );
      psp[3].dwFlags     = PSP_USETITLE;
      psp[3].hInstance   = hInst;
      psp[3].pszTemplate = MAKEINTRESOURCE( IDD_CP2REGS_CORE );
      psp[3].pszIcon     = NULL;
      psp[3].pfnDlgProc  = (DLGPROC)CP2RegsCore;
      psp[3].pszTitle    = "RCP Core";
      psp[3].lParam      = 0;

      // Define the Vector CP2 Register Debug Page

      psp[4].dwSize      = sizeof( PROPSHEETPAGE );
      psp[4].dwFlags     = PSP_USETITLE;
      psp[4].hInstance   = hInst;
      psp[4].pszTemplate = MAKEINTRESOURCE( IDD_CP2REGS_VECTOR );
      psp[4].pszIcon     = NULL;
      psp[4].pfnDlgProc  = (DLGPROC)CP2RegsVector;
      psp[4].pszTitle    = "RCP Vector";
      psp[4].lParam      = 0;

      // Define the VI/AI/DPS/SI Register Debug Page

      psp[5].dwSize      = sizeof( PROPSHEETPAGE );
      psp[5].dwFlags     = PSP_USETITLE;
      psp[5].hInstance   = hInst;
      psp[5].pszTemplate = MAKEINTRESOURCE( IDD_VIAIDPSSIREGS );
      psp[5].pszIcon     = NULL;
      psp[5].pfnDlgProc  = (DLGPROC)ViAiDpsSiRegs;
      psp[5].pszTitle    = "VI/AI/DPS/SI";
      psp[5].lParam      = 0;

      // Define the PI/RI Register Debug Page

      psp[6].dwSize      = sizeof( PROPSHEETPAGE );
      psp[6].dwFlags     = PSP_USETITLE;
      psp[6].hInstance   = hInst;
      psp[6].pszTemplate = MAKEINTRESOURCE( IDD_PIRIREGS );
      psp[6].pszIcon     = NULL;
      psp[6].pfnDlgProc  = (DLGPROC)PiRiRegs;
      psp[6].pszTitle    = "PI/RI";
      psp[6].lParam      = 0;

      // Define Property Sheet Header

      psh.dwSize     = sizeof( PROPSHEETHEADER );
      psh.dwFlags    = PSH_PROPSHEETPAGE | PSH_NOAPPLYNOW | 
                       PSH_USEICONID | PSH_MODELESS;
      psh.hwndParent = NULL;
      psh.hInstance  = hInst;
      psh.pszIcon    = MAKEINTRESOURCE( IDI_DEBUG );
      psh.pszCaption = (LPSTR)"Internal Registers";
      psh.nPages     = sizeof( psp ) / sizeof( PROPSHEETPAGE );;
      psh.nStartPage = 0;
      psh.ppsp       = (LPCPROPSHEETPAGE) &psp;

      return( PropertySheet( &psh ) );
   }


////////////////////////////////////////////////////////////////////////////////
// General Purpose Registers - Tabbed Dialog

   BOOL APIENTRY DebugGPRegs( HWND hDlg, UINT message, 
                              UINT wParam, LONG lParam )
   {
      static PROPSHEETPAGE *ps;        // Pointer to Property Sheet Information

      switch( message )
      {
         case WM_INITDIALOG:

            // Save the PROPSHEETPAGE Information

            ps = (PROPSHEETPAGE *)lParam;

            hwndDebugGPRegs = hDlg;

            return( TRUE );

            break;

         case WM_COMMAND:

            switch( LOWORD( wParam ) )
            {
               case ID_REFRESH:

                  // General Purpose Registers

                  wsprintf( szHex, "0x%016X\0\0", gpReg[0] );
                  SetDlgItemText( hDlg, IDC_GP_R0, szHex );
                  wsprintf( szHex, "0x%016X\0\0", gpReg[1] );
                  SetDlgItemText( hDlg, IDC_GP_AT, szHex );
                  wsprintf( szHex, "0x%016X\0\0", gpReg[2] );
                  SetDlgItemText( hDlg, IDC_GP_V0, szHex );
                  wsprintf( szHex, "0x%016X\0\0", gpReg[3] );
                  SetDlgItemText( hDlg, IDC_GP_V1, szHex );
                  wsprintf( szHex, "0x%016X\0\0", gpReg[4] );
                  SetDlgItemText( hDlg, IDC_GP_A0, szHex );
                  wsprintf( szHex, "0x%016X\0\0", gpReg[5] );
                  SetDlgItemText( hDlg, IDC_GP_A1, szHex );
                  wsprintf( szHex, "0x%016X\0\0", gpReg[6] );
                  SetDlgItemText( hDlg, IDC_GP_A2, szHex );
                  wsprintf( szHex, "0x%016X\0\0", gpReg[7] );
                  SetDlgItemText( hDlg, IDC_GP_A3, szHex );

                  wsprintf( szHex, "%#016X\0", gpReg[8] );
                  SetDlgItemText( hDlg, IDC_GP_T0, szHex );

                  wsprintf( szHex, "0x%016X\0", gpReg[9] );
                  SetDlgItemText( hDlg, IDC_GP_T1, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[10] );
                  SetDlgItemText( hDlg, IDC_GP_T2, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[11] );
                  SetDlgItemText( hDlg, IDC_GP_T3, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[12] );
                  SetDlgItemText( hDlg, IDC_GP_T4, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[13] );
                  SetDlgItemText( hDlg, IDC_GP_T5, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[14] );
                  SetDlgItemText( hDlg, IDC_GP_T6, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[15] );
                  SetDlgItemText( hDlg, IDC_GP_T7, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[16] );
                  SetDlgItemText( hDlg, IDC_GP_S0, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[17] );
                  SetDlgItemText( hDlg, IDC_GP_S1, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[18] );
                  SetDlgItemText( hDlg, IDC_GP_S2, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[19] );
                  SetDlgItemText( hDlg, IDC_GP_S3, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[20] );
                  SetDlgItemText( hDlg, IDC_GP_S4, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[21] );
                  SetDlgItemText( hDlg, IDC_GP_S5, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[22] );
                  SetDlgItemText( hDlg, IDC_GP_S6, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[23] );
                  SetDlgItemText( hDlg, IDC_GP_S7, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[24] );
                  SetDlgItemText( hDlg, IDC_GP_T8, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[25] );
                  SetDlgItemText( hDlg, IDC_GP_T9, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[26] );
                  SetDlgItemText( hDlg, IDC_GP_K0, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[27] );
                  SetDlgItemText( hDlg, IDC_GP_K1, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[28] );
                  SetDlgItemText( hDlg, IDC_GP_GP, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[29] );
                  SetDlgItemText( hDlg, IDC_GP_SP, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[30] );
                  SetDlgItemText( hDlg, IDC_GP_S8, szHex );
                  wsprintf( szHex, "0x%016X\0", gpReg[31] );
                  SetDlgItemText( hDlg, IDC_GP_RA, szHex );

                  // Program Counter

                  wsprintf( szHex, "0x%08X\0", PC );
                  SetDlgItemText( hDlg, IDC_PC, szHex );

                  // Instruction Word

                  wsprintf( szHex, "0x%08X\0", instrWord );
                  SetDlgItemText( hDlg, IDC_INSTRWORD, szHex );

                  // HI Register

                  wsprintf( szHex, "0x%016X\0", HI );
                  SetDlgItemText( hDlg, IDC_HI, szHex );

                  // LO Register

                  wsprintf( szHex, "0x%016X\0", LO );
                  SetDlgItemText( hDlg, IDC_LO, szHex );

                  // LLBit

                  wsprintf( szHex, "%01X\0", LLBit );
                  SetDlgItemText( hDlg, IDC_LLBIT, szHex );

                  // MIPS Interface registers

                  wsprintf( szHex, "0x%08X\0", *(miInitModeReg) );
                  SetDlgItemText( hDlg, IDC_MI_INITMODE, szHex );
                  wsprintf( szHex, "0x%08X\0", *(miVersionReg) );
                  SetDlgItemText( hDlg, IDC_MI_VERSION, szHex );
                  wsprintf( szHex, "0x%08X\0", *(miIntrReg) );
                  SetDlgItemText( hDlg, IDC_MI_INTR, szHex );
                  wsprintf( szHex, "0x%08X\0", *(miIntrMaskReg) );
                  SetDlgItemText( hDlg, IDC_MI_INTRMASK, szHex );

                  return( TRUE );

                  break;
            }

            break;

         case WM_NOTIFY:

            switch( ((NMHDR FAR *)lParam)->code )
            {
               case PSN_SETACTIVE:

                  SendMessage( hDlg, WM_COMMAND, ID_REFRESH, 0L );

                  break;

               case PSN_APPLY:

                  SetWindowLong( hDlg, DWL_MSGRESULT, TRUE );

                  break;

               case PSN_KILLACTIVE:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );

                  return( TRUE );

                  break;

               case PSN_RESET:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );

                  break;

               default:

                  break;
            }

            break;
      }

      return( FALSE );
   }


////////////////////////////////////////////////////////////////////////////////
// System Control Registers (COP0) - Tabbed Dialog

   BOOL APIENTRY CP0Regs( HWND hDlg, UINT message, 
                          UINT wParam, LONG lParam )
   {
      static PROPSHEETPAGE *ps;        // Pointer to Property Sheet Information

      switch( message )
      {
         case WM_INITDIALOG:

            // Save the PROPSHEETPAGE Information

            ps = (PROPSHEETPAGE *)lParam;

            hwndCP0Regs = hDlg;

            return( TRUE );

            break;

         case WM_COMMAND:

            switch( LOWORD( wParam ) )
            {
               case ID_REFRESH:

                  // CP0 (System Control Co-processor) Registers

                  wsprintf( szHex, "0x%08X\0", cp0Reg[0] );
                  SetDlgItemText( hDlg, IDC_INDEX, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[1] );
                  SetDlgItemText( hDlg, IDC_RANDOM, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[2] );
                  SetDlgItemText( hDlg, IDC_ENTRYLO0, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[3] );
                  SetDlgItemText( hDlg, IDC_ENTRYLO1, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[4] );
                  SetDlgItemText( hDlg, IDC_CONTEXT, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[5] );
                  SetDlgItemText( hDlg, IDC_PAGEMASK, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[6] );
                  SetDlgItemText( hDlg, IDC_WIRED, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[8] );
                  SetDlgItemText( hDlg, IDC_BADVADDR, szHex );
                  wsprintf( szHex, "0x%016X\0", cp0Reg[9] );
                  SetDlgItemText( hDlg, IDC_COUNT, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[10] );
                  SetDlgItemText( hDlg, IDC_ENTRYHI, szHex );
                  wsprintf( szHex, "0x%016X\0", cp0Reg[11] );
                  SetDlgItemText( hDlg, IDC_COMPARE, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[12] );
                  SetDlgItemText( hDlg, IDC_STATUS, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[13] );
                  SetDlgItemText( hDlg, IDC_CAUSE, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[14] );
                  SetDlgItemText( hDlg, IDC_EPC, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[15] );
                  SetDlgItemText( hDlg, IDC_PRID, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[16] );
                  SetDlgItemText( hDlg, IDC_CONFIG, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[17] );
                  SetDlgItemText( hDlg, IDC_LLADDR, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[18] );
                  SetDlgItemText( hDlg, IDC_WATCHLO, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[19] );
                  SetDlgItemText( hDlg, IDC_WATCHHI, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[20] );
                  SetDlgItemText( hDlg, IDC_XCONTEXT, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[26] );
                  SetDlgItemText( hDlg, IDC_PARITYERROR, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[27] );
                  SetDlgItemText( hDlg, IDC_CACHEERROR, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[28] );
                  SetDlgItemText( hDlg, IDC_TAGLO, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[29] );
                  SetDlgItemText( hDlg, IDC_TAGHI, szHex );
                  wsprintf( szHex, "0x%08X\0", cp0Reg[30] );
                  SetDlgItemText( hDlg, IDC_ERROREPC, szHex );

                  return( TRUE );

                  break;
            }

            break;

         case WM_NOTIFY:

            switch( ((NMHDR FAR *)lParam)->code )
            {
               case PSN_SETACTIVE:

                  SendMessage( hDlg, WM_COMMAND, ID_REFRESH, 0L );

                  break;

               case PSN_APPLY:

                  SetWindowLong( hDlg, DWL_MSGRESULT, TRUE );

                  break;

               case PSN_KILLACTIVE:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );

                  return( TRUE );

                  break;

               case PSN_RESET:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );

                  break;

               default:

                  break;
            }

            break;
      }

      return( FALSE );
   }

////////////////////////////////////////////////////////////////////////////////
// Floating Point Registers (COP1) - Tabbed Dialog

   BOOL APIENTRY CP1Regs( HWND hDlg, UINT message, 
                          UINT wParam, LONG lParam )
   {
      static PROPSHEETPAGE *ps;        // Pointer to Property Sheet Information

      switch( message )
      {
         case WM_INITDIALOG:

            // Save the PROPSHEETPAGE Information

            ps = (PROPSHEETPAGE *)lParam;

            hwndCP1Regs = hDlg;

            return( TRUE );

            break;

         case WM_COMMAND:

            switch( LOWORD( wParam ) )
            {
               case ID_REFRESH:

                  // CP1 (Floating Point Co-processor) Registers

                  wsprintf( szHex, "0x%016X\0", cp1Reg[0] );
                  SetDlgItemText( hDlg, IDC_FP0, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[1] );
                  SetDlgItemText( hDlg, IDC_FP1, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[2] );
                  SetDlgItemText( hDlg, IDC_FP2, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[3] );
                  SetDlgItemText( hDlg, IDC_FP3, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[4] );
                  SetDlgItemText( hDlg, IDC_FP4, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[5] );
                  SetDlgItemText( hDlg, IDC_FP5, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[6] );
                  SetDlgItemText( hDlg, IDC_FP6, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[7] );
                  SetDlgItemText( hDlg, IDC_FP7, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[8] );
                  SetDlgItemText( hDlg, IDC_FP8, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[9] );
                  SetDlgItemText( hDlg, IDC_FP9, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[10] );
                  SetDlgItemText( hDlg, IDC_FP10, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[11] );
                  SetDlgItemText( hDlg, IDC_FP11, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[12] );
                  SetDlgItemText( hDlg, IDC_FP12, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[13] );
                  SetDlgItemText( hDlg, IDC_FP13, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[14] );
                  SetDlgItemText( hDlg, IDC_FP14, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[15] );
                  SetDlgItemText( hDlg, IDC_FP15, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[16] );
                  SetDlgItemText( hDlg, IDC_FP16, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[17] );
                  SetDlgItemText( hDlg, IDC_FP17, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[18] );
                  SetDlgItemText( hDlg, IDC_FP18, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[19] );
                  SetDlgItemText( hDlg, IDC_FP19, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[20] );
                  SetDlgItemText( hDlg, IDC_FP20, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[21] );
                  SetDlgItemText( hDlg, IDC_FP21, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[22] );
                  SetDlgItemText( hDlg, IDC_FP22, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[23] );
                  SetDlgItemText( hDlg, IDC_FP23, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[24] );
                  SetDlgItemText( hDlg, IDC_FP24, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[25] );
                  SetDlgItemText( hDlg, IDC_FP25, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[26] );
                  SetDlgItemText( hDlg, IDC_FP26, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[27] );
                  SetDlgItemText( hDlg, IDC_FP27, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[28] );
                  SetDlgItemText( hDlg, IDC_FP28, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[29] );
                  SetDlgItemText( hDlg, IDC_FP29, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[30] );
                  SetDlgItemText( hDlg, IDC_FP30, szHex );
                  wsprintf( szHex, "0x%016X\0", cp1Reg[31] );
                  SetDlgItemText( hDlg, IDC_FP31, szHex );

                  // FCR0 Register

                  wsprintf( szHex, "0x%08X\0", FCR0 );
                  SetDlgItemText( hDlg, IDC_FCR0, szHex );

                  // FCR31 Register

                  wsprintf( szHex, "0x%08X\0", FCR31 );
                  SetDlgItemText( hDlg, IDC_FCR31, szHex );

                  return( TRUE );

                  break;
            }

            break;

         case WM_NOTIFY:

            switch( ((NMHDR FAR *)lParam)->code )
            {
               case PSN_SETACTIVE:

                  SendMessage( hDlg, WM_COMMAND, ID_REFRESH, 0L );

                  break;

               case PSN_APPLY:

                  SetWindowLong( hDlg, DWL_MSGRESULT, TRUE );

                  break;

               case PSN_KILLACTIVE:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );

                  return( TRUE );

                  break;

               case PSN_RESET:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );

                  break;

               default:

                  break;
            }

            break;
      }

      return( FALSE );
   }

////////////////////////////////////////////////////////////////////////////////
// Core RCP Registers (COP2) - Tabbed Dialog

   BOOL APIENTRY CP2RegsCore( HWND hDlg, UINT message, 
                              UINT wParam, LONG lParam )
   {
      static PROPSHEETPAGE *ps;        // Pointer to Property Sheet Information

      switch( message )
      {
         case WM_INITDIALOG:

            // Save the PROPSHEETPAGE Information

            ps = (PROPSHEETPAGE *)lParam;

            hwndCP2RegsCore = hDlg;

            return( TRUE );
            break;

         case WM_COMMAND:

            switch( LOWORD( wParam ) )
            {
               case ID_REFRESH:

                  // CP2 (Core RCP) Registers

                  wsprintf( szHex, "0x%016X\0", cp2Reg[0] );
                  SetDlgItemText( hDlg, IDC_CP2_R0, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[1] );
                  SetDlgItemText( hDlg, IDC_CP2_AT, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[2] );
                  SetDlgItemText( hDlg, IDC_CP2_V0, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[3] );
                  SetDlgItemText( hDlg, IDC_CP2_V1, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[4] );
                  SetDlgItemText( hDlg, IDC_CP2_A0, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[5] );
                  SetDlgItemText( hDlg, IDC_CP2_A1, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[6] );
                  SetDlgItemText( hDlg, IDC_CP2_A2, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[7] );
                  SetDlgItemText( hDlg, IDC_CP2_A3, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[8] );
                  SetDlgItemText( hDlg, IDC_CP2_T0, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[9] );
                  SetDlgItemText( hDlg, IDC_CP2_T1, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[10] );
                  SetDlgItemText( hDlg, IDC_CP2_T2, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[11] );
                  SetDlgItemText( hDlg, IDC_CP2_T3, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[12] );
                  SetDlgItemText( hDlg, IDC_CP2_T4, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[13] );
                  SetDlgItemText( hDlg, IDC_CP2_T5, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[14] );
                  SetDlgItemText( hDlg, IDC_CP2_T6, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[15] );
                  SetDlgItemText( hDlg, IDC_CP2_T7, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[16] );
                  SetDlgItemText( hDlg, IDC_CP2_S0, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[17] );
                  SetDlgItemText( hDlg, IDC_CP2_S1, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[18] );
                  SetDlgItemText( hDlg, IDC_CP2_S2, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[19] );
                  SetDlgItemText( hDlg, IDC_CP2_S3, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[20] );
                  SetDlgItemText( hDlg, IDC_CP2_S4, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[21] );
                  SetDlgItemText( hDlg, IDC_CP2_S5, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[22] );
                  SetDlgItemText( hDlg, IDC_CP2_S6, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[23] );
                  SetDlgItemText( hDlg, IDC_CP2_S7, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[24] );
                  SetDlgItemText( hDlg, IDC_CP2_T8, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[25] );
                  SetDlgItemText( hDlg, IDC_CP2_T9, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[26] );
                  SetDlgItemText( hDlg, IDC_CP2_K0, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[27] );
                  SetDlgItemText( hDlg, IDC_CP2_K1, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[28] );
                  SetDlgItemText( hDlg, IDC_CP2_GP, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[29] );
                  SetDlgItemText( hDlg, IDC_CP2_SP, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[30] );
                  SetDlgItemText( hDlg, IDC_CP2_S8, szHex );
                  wsprintf( szHex, "0x%016X\0", cp2Reg[31] );
                  SetDlgItemText( hDlg, IDC_CP2_RA, szHex );

                  // SP CP0 Registers

                  wsprintf( szHex, "0x%08X\0", *(spMemAddrReg) );
                  SetDlgItemText( hDlg, IDC_SP_MEMADDR, szHex );
                  wsprintf( szHex, "0x%08X\0", *(spDramAddrReg) );
                  SetDlgItemText( hDlg, IDC_SP_DRAMADDR, szHex );
                  wsprintf( szHex, "0x%08X\0", *(spRdLenReg) );
                  SetDlgItemText( hDlg, IDC_SP_READLEN, szHex );
                  wsprintf( szHex, "0x%08X\0", *(spWrLenReg) );
                  SetDlgItemText( hDlg, IDC_SP_WRITELEN, szHex );
                  wsprintf( szHex, "0x%08X\0", *(spStatusReg) );
                  SetDlgItemText( hDlg, IDC_SP_STATUS, szHex );
                  wsprintf( szHex, "0x%08X\0", *(spDmaFullReg) );
                  SetDlgItemText( hDlg, IDC_SP_DMAFULL, szHex );
                  wsprintf( szHex, "0x%08X\0", *(spDmaBusyReg) );
                  SetDlgItemText( hDlg, IDC_SP_DMABUSY, szHex );
                  wsprintf( szHex, "0x%08X\0", *(spSemaphoreReg) );
                  SetDlgItemText( hDlg, IDC_SP_SEMAPHORE, szHex );

                  // DP Command Registers

                  wsprintf( szHex, "0x%08X\0", *(dpcStartReg) );
                  SetDlgItemText( hDlg, IDC_DPC_DMASTART, szHex );
                  wsprintf( szHex, "0x%08X\0", *(dpcEndReg) );
                  SetDlgItemText( hDlg, IDC_DPC_DMAEND, szHex );
                  wsprintf( szHex, "0x%08X\0", *(dpcCurrentReg) );
                  SetDlgItemText( hDlg, IDC_DPC_CURRENT, szHex );
                  wsprintf( szHex, "0x%08X\0", *(dpcStatusReg) );
                  SetDlgItemText( hDlg, IDC_DPC_STATUS, szHex );
                  wsprintf( szHex, "0x%08X\0", *(dpcClockReg) );
                  SetDlgItemText( hDlg, IDC_DPC_CLOCK, szHex );
                  wsprintf( szHex, "0x%08X\0", *(dpcBufBusyReg) );
                  SetDlgItemText( hDlg, IDC_DPC_BUFBUSY, szHex );
                  wsprintf( szHex, "0x%08X\0", *(dpcPipeBusyReg) );
                  SetDlgItemText( hDlg, IDC_DPC_PIPEBUSY, szHex );
                  wsprintf( szHex, "0x%08X\0", *(dpcTmemReg) );
                  SetDlgItemText( hDlg, IDC_DPC_TMEM, szHex );

                  return( TRUE );

                  break;
            }

            break;

         case WM_NOTIFY:

            switch( ((NMHDR FAR *)lParam)->code )
            {
               case PSN_SETACTIVE:

                  SendMessage( hDlg, WM_COMMAND, ID_REFRESH, 0L );

                  break;

               case PSN_APPLY:

                  SetWindowLong( hDlg, DWL_MSGRESULT, TRUE );
                   
                  break;

               case PSN_KILLACTIVE:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );

                  return( TRUE );

                  break;

               case PSN_RESET:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );

                  break;

               default:

                  break;
            }

            break;
      }

      return( FALSE );
   }


////////////////////////////////////////////////////////////////////////////////
// Vector RCP Registers (COP2) - Tabbed Dialog

   BOOL APIENTRY CP2RegsVector( HWND hDlg, UINT message, 
                                UINT wParam, LONG lParam )
   {
      static PROPSHEETPAGE *ps;        // Pointer to Property Sheet Information

      switch( message )
      {
         case WM_INITDIALOG:

            // Save the PROPSHEETPAGE Information

            ps = (PROPSHEETPAGE *)lParam;

            hwndCP2RegsVector = hDlg;

            return( TRUE );

            break;

         case WM_COMMAND:

            switch( LOWORD( wParam ) )
            {
               case ID_REFRESH:

                  // CP2 (Vector) Registers

                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[0].e7, vReg[0].e6, vReg[0].e5,
                                   vReg[0].e4, vReg[0].e3, vReg[0].e2,
                                   vReg[0].e1, vReg[0].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_0, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[1].e7, vReg[1].e6, vReg[1].e5,
                                   vReg[1].e4, vReg[1].e3, vReg[1].e2,
                                   vReg[1].e1, vReg[1].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_1, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[2].e7, vReg[2].e6, vReg[2].e5,
                                   vReg[2].e4, vReg[2].e3, vReg[2].e2,
                                   vReg[2].e1, vReg[2].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_2, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[3].e7, vReg[3].e6, vReg[3].e5,
                                   vReg[3].e4, vReg[3].e3, vReg[3].e2,
                                   vReg[3].e1, vReg[3].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_3, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[4].e7, vReg[4].e6, vReg[4].e5,
                                   vReg[4].e4, vReg[4].e3, vReg[4].e2,
                                   vReg[4].e1, vReg[4].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_4, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[5].e7, vReg[5].e6, vReg[5].e5,
                                   vReg[5].e4, vReg[5].e3, vReg[5].e2,
                                   vReg[5].e1, vReg[5].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_5, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[6].e7, vReg[6].e6, vReg[6].e5,
                                   vReg[6].e4, vReg[6].e3, vReg[6].e2,
                                   vReg[6].e1, vReg[6].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_6, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[7].e7, vReg[7].e6, vReg[7].e5,
                                   vReg[7].e4, vReg[7].e3, vReg[7].e2,
                                   vReg[7].e1, vReg[7].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_7, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[8].e7, vReg[8].e6, vReg[8].e5,
                                   vReg[8].e4, vReg[8].e3, vReg[8].e2,
                                   vReg[8].e1, vReg[8].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_8, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[9].e7, vReg[9].e6, vReg[9].e5,
                                   vReg[9].e4, vReg[9].e3, vReg[9].e2,
                                   vReg[9].e1, vReg[9].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_9, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[10].e7, vReg[10].e6, vReg[10].e5,
                                   vReg[10].e4, vReg[10].e3, vReg[10].e2,
                                   vReg[10].e1, vReg[10].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_10, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[11].e7, vReg[11].e6, vReg[11].e5,
                                   vReg[11].e4, vReg[11].e3, vReg[11].e2,
                                   vReg[11].e1, vReg[11].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_11, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[12].e7, vReg[12].e6, vReg[12].e5,
                                   vReg[12].e4, vReg[12].e3, vReg[12].e2,
                                   vReg[12].e1, vReg[12].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_12, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[13].e7, vReg[13].e6, vReg[13].e5,
                                   vReg[13].e4, vReg[13].e3, vReg[13].e2,
                                   vReg[13].e1, vReg[13].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_13, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[14].e7, vReg[14].e6, vReg[14].e5,
                                   vReg[14].e4, vReg[14].e3, vReg[14].e2,
                                   vReg[14].e1, vReg[14].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_14, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[15].e7, vReg[15].e6, vReg[15].e5,
                                   vReg[15].e4, vReg[15].e3, vReg[15].e2,
                                   vReg[15].e1, vReg[15].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_15, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[16].e7, vReg[16].e6, vReg[16].e5,
                                   vReg[16].e4, vReg[16].e3, vReg[16].e2,
                                   vReg[16].e1, vReg[16].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_16, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[17].e7, vReg[17].e6, vReg[17].e5,
                                   vReg[17].e4, vReg[17].e3, vReg[17].e2,
                                   vReg[17].e1, vReg[17].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_17, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[18].e7, vReg[18].e6, vReg[18].e5,
                                   vReg[18].e4, vReg[18].e3, vReg[18].e2,
                                   vReg[18].e1, vReg[18].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_18, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[19].e7, vReg[19].e6, vReg[19].e5,
                                   vReg[19].e4, vReg[19].e3, vReg[19].e2,
                                   vReg[19].e1, vReg[19].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_19, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[20].e7, vReg[20].e6, vReg[20].e5,
                                   vReg[20].e4, vReg[20].e3, vReg[20].e2,
                                   vReg[20].e1, vReg[20].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_20, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[21].e7, vReg[21].e6, vReg[21].e5,
                                   vReg[21].e4, vReg[21].e3, vReg[21].e2,
                                   vReg[21].e1, vReg[21].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_21, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[22].e7, vReg[22].e6, vReg[22].e5,
                                   vReg[22].e4, vReg[22].e3, vReg[22].e2,
                                   vReg[22].e1, vReg[22].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_22, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[23].e7, vReg[23].e6, vReg[23].e5,
                                   vReg[23].e4, vReg[23].e3, vReg[23].e2,
                                   vReg[23].e1, vReg[23].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_23, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[24].e7, vReg[24].e6, vReg[24].e5,
                                   vReg[24].e4, vReg[24].e3, vReg[24].e2,
                                   vReg[24].e1, vReg[24].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_24, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[25].e7, vReg[25].e6, vReg[25].e5,
                                   vReg[25].e4, vReg[25].e3, vReg[25].e2,
                                   vReg[25].e1, vReg[25].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_25, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[26].e7, vReg[26].e6, vReg[26].e5,
                                   vReg[26].e4, vReg[26].e3, vReg[26].e2,
                                   vReg[26].e1, vReg[26].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_26, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[27].e7, vReg[27].e6, vReg[27].e5,
                                   vReg[27].e4, vReg[27].e3, vReg[27].e2,
                                   vReg[27].e1, vReg[27].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_27, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[28].e7, vReg[28].e6, vReg[28].e5,
                                   vReg[28].e4, vReg[28].e3, vReg[28].e2,
                                   vReg[28].e1, vReg[28].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_28, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[29].e7, vReg[29].e6, vReg[29].e5,
                                   vReg[29].e4, vReg[29].e3, vReg[29].e2,
                                   vReg[29].e1, vReg[29].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_29, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[30].e7, vReg[30].e6, vReg[30].e5,
                                   vReg[30].e4, vReg[30].e3, vReg[30].e2,
                                   vReg[30].e1, vReg[30].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_30, szHex );
                  wsprintf( szHex, "0x%04X, 0x%04X, 0x%04X, 0x%04X, "
                                   "0x%04X, 0x%04X, 0x%04X, 0x%04X\0", 
                                   vReg[31].e7, vReg[31].e6, vReg[31].e5,
                                   vReg[31].e4, vReg[31].e3, vReg[31].e2,
                                   vReg[31].e1, vReg[31].e0 );
                  SetDlgItemText( hDlg, IDC_VECTOR_31, szHex );

                  return( TRUE );

                  break;
            }

            break;

         case WM_NOTIFY:

            switch( ((NMHDR FAR *)lParam)->code )
            {
               case PSN_SETACTIVE:

                  SendMessage( hDlg, WM_COMMAND, ID_REFRESH, 0L );

                  break;

               case PSN_APPLY:

                  SetWindowLong( hDlg, DWL_MSGRESULT, TRUE );

                  break;

               case PSN_KILLACTIVE:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );

                  return( TRUE );

                  break;

               case PSN_RESET:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );

                  break;

               default:

                  break;
            }

            break;
      }

      return( FALSE );
   }


////////////////////////////////////////////////////////////////////////////////
// VI/AI/DPS/SI Registers - Tabbed Dialog

   BOOL APIENTRY ViAiDpsSiRegs( HWND hDlg, UINT message, 
                                UINT wParam, LONG lParam )
   {
      static PROPSHEETPAGE *ps;        // Pointer to Property Sheet Information

      switch( message )
      {
         case WM_INITDIALOG:

            // Save the PROPSHEETPAGE Information

            ps = (PROPSHEETPAGE *)lParam;

            hwndViAiDpsSiRegs = hDlg;

            return( TRUE );

            break;

         case WM_COMMAND:

            switch( LOWORD( wParam ) )
            {
               case ID_REFRESH:

                  // Video Interface (VI) Registers

                  wsprintf( szHex, "0x%08X\0", *(viStatusReg) );
                  SetDlgItemText( hDlg, IDC_VI_STATUS, szHex );
                  wsprintf( szHex, "0x%08X\0", *(viOriginReg) );
                  SetDlgItemText( hDlg, IDC_VI_ORIGIN, szHex );
                  wsprintf( szHex, "0x%08X\0", *(viWidthReg) );
                  SetDlgItemText( hDlg, IDC_VI_WIDTH, szHex );
                  wsprintf( szHex, "0x%08X\0", *(viIntrReg) );
                  SetDlgItemText( hDlg, IDC_VI_VERTINTR, szHex );
                  wsprintf( szHex, "0x%08X\0", *(viCurrentReg) );
                  SetDlgItemText( hDlg, IDC_VI_CURRVERT, szHex );
                  wsprintf( szHex, "0x%08X\0", *(viBurstReg) );
                  SetDlgItemText( hDlg, IDC_VI_VIDEOTIME, szHex );
                  wsprintf( szHex, "0x%08X\0", *(viVSyncReg) );
                  SetDlgItemText( hDlg, IDC_VI_VERTSYNC, szHex );
                  wsprintf( szHex, "0x%08X\0", *(viHSyncReg) );
                  SetDlgItemText( hDlg, IDC_VI_HORIZSYNC, szHex );
                  wsprintf( szHex, "0x%08X\0", *(viLeapReg) );
                  SetDlgItemText( hDlg, IDC_VI_LEAP, szHex );
                  wsprintf( szHex, "0x%08X\0", *(viHStartReg) );
                  SetDlgItemText( hDlg, IDC_VI_HORIZVID, szHex );
                  wsprintf( szHex, "0x%08X\0", *(viVStartReg) );
                  SetDlgItemText( hDlg, IDC_VI_VERTVID, szHex );
                  wsprintf( szHex, "0x%08X\0", *(viVBurstReg) );
                  SetDlgItemText( hDlg, IDC_VI_VERTBURST, szHex );
                  wsprintf( szHex, "0x%08X\0", *(viXScaleReg) );
                  SetDlgItemText( hDlg, IDC_VI_XSCALE, szHex );
                  wsprintf( szHex, "0x%08X\0", *(viYScaleReg) );
                  SetDlgItemText( hDlg, IDC_VI_YSCALE, szHex );

                  // Audio Interface (AI) Registers

                  wsprintf( szHex, "0x%08X\0", *(aiDramAddrReg) );
                  SetDlgItemText( hDlg, IDC_AI_DRAMADDR, szHex );
                  wsprintf( szHex, "0x%08X\0", *(aiLenReg) );
                  SetDlgItemText( hDlg, IDC_AI_LENGTH, szHex );
                  wsprintf( szHex, "0x%08X\0", *(aiControlReg) );
                  SetDlgItemText( hDlg, IDC_AI_CONTROL, szHex );
                  wsprintf( szHex, "0x%08X\0", *(aiStatusReg) );
                  SetDlgItemText( hDlg, IDC_AI_STATUS, szHex );
                  wsprintf( szHex, "0x%08X\0", *(aiDacRateReg) );
                  SetDlgItemText( hDlg, IDC_AI_DACRATE, szHex );
                  wsprintf( szHex, "0x%08X\0", *(aiBitRateReg) );
                  SetDlgItemText( hDlg, IDC_AI_BITRATE, szHex );

                  // DP Span Registers

                  wsprintf( szHex, "0x%08X\0", *(dpsTbistReg) );
                  SetDlgItemText( hDlg, IDC_DPS_TBIST, szHex );
                  wsprintf( szHex, "0x%08X\0", *(dpsTestModeReg) );
                  SetDlgItemText( hDlg, IDC_DPS_TESTMDE, szHex );
                  wsprintf( szHex, "0x%08X\0", *(dpsBufTestAddrReg) );
                  SetDlgItemText( hDlg, IDC_DPS_TESTADDR, szHex );
                  wsprintf( szHex, "0x%08X\0", *(dpsBufTestDataReg) );
                  SetDlgItemText( hDlg, IDC_DPS_TESTDATA, szHex );

                  // Serial Interface (SI) Registers

                  wsprintf( szHex, "0x%08X\0", *(siDramAddrReg) );
                  SetDlgItemText( hDlg, IDC_SI_DRAMADDR, szHex );
                  wsprintf( szHex, "0x%08X\0", *(siPifAddrRd64bReg) );
                  SetDlgItemText( hDlg, IDC_SI_PIFREAD, szHex );
                  wsprintf( szHex, "0x%08X\0", *(siPifAddrWr64bReg) );
                  SetDlgItemText( hDlg, IDC_SI_PIFWRITE, szHex );
                  wsprintf( szHex, "0x%08X\0", *(siStatusReg) );
                  SetDlgItemText( hDlg, IDC_SI_STATUS, szHex );

                  return( TRUE );

                  break;
            }

            break;

         case WM_NOTIFY:

            switch( ((NMHDR FAR *)lParam)->code )
            {
               case PSN_SETACTIVE:

                  SendMessage( hDlg, WM_COMMAND, ID_REFRESH, 0L );

                  break;

               case PSN_APPLY:

                  SetWindowLong( hDlg, DWL_MSGRESULT, TRUE );

                  break;

               case PSN_KILLACTIVE:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );

                  return( TRUE );

                  break;

               case PSN_RESET:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );

                  break;

               default:

                  break;
            }

            break;
      }

      return( FALSE );
   }


////////////////////////////////////////////////////////////////////////////////
// PI/RI Registers - Tabbed Dialog

   BOOL APIENTRY PiRiRegs( HWND hDlg, UINT message, 
                           UINT wParam, LONG lParam )
   {
      static PROPSHEETPAGE *ps;        // Pointer to Property Sheet Information

      switch( message )
      {
         case WM_INITDIALOG:

            // Save the PROPSHEETPAGE Information

            ps = (PROPSHEETPAGE *)lParam;

            hwndPiRiRegs = hDlg;

            return( TRUE );

            break;

         case WM_COMMAND:

            switch( LOWORD( wParam ) )
            {
               case ID_REFRESH:

                  // Peripheral Interface (PI) Registers

                  wsprintf( szHex, "0x%08X\0", *(piDramAddrReg) );
                  SetDlgItemText( hDlg, IDC_PI_DRAMADDR, szHex );
                  wsprintf( szHex, "0x%08X\0", *(piCartAddrReg) );
                  SetDlgItemText( hDlg, IDC_PI_CARTADDR, szHex );
                  wsprintf( szHex, "0x%08X\0", *(piRdLenReg) );
                  SetDlgItemText( hDlg, IDC_PI_READLEN, szHex );
                  wsprintf( szHex, "0x%08X\0", *(piWrLenReg) );
                  SetDlgItemText( hDlg, IDC_PI_WRITELEN, szHex );
                  wsprintf( szHex, "0x%08X\0", *(piStatusReg) );
                  SetDlgItemText( hDlg, IDC_PI_STATUS, szHex );
                  wsprintf( szHex, "0x%08X\0", *(piBsdDom1LatReg) );
                  SetDlgItemText( hDlg, IDC_PI_DOM1LAT, szHex );
                  wsprintf( szHex, "0x%08X\0", *(piBsdDom1PwdReg) );
                  SetDlgItemText( hDlg, IDC_PI_DOM1PWD, szHex );
                  wsprintf( szHex, "0x%08X\0", *(piBsdDom1PgsReg) );
                  SetDlgItemText( hDlg, IDC_PI_DOM1PGS, szHex );
                  wsprintf( szHex, "0x%08X\0", *(piBsdDom1RlsReg) );
                  SetDlgItemText( hDlg, IDC_PI_DOM1RLS, szHex );
                  wsprintf( szHex, "0x%08X\0", *(piBsdDom2LatReg) );
                  SetDlgItemText( hDlg, IDC_PI_DOM2LAT, szHex );
                  wsprintf( szHex, "0x%08X\0", *(piBsdDom2PwdReg) );
                  SetDlgItemText( hDlg, IDC_PI_DOM2PWD, szHex );
                  wsprintf( szHex, "0x%08X\0", *(piBsdDom2PgsReg) );
                  SetDlgItemText( hDlg, IDC_PI_DOM2PGS, szHex );
                  wsprintf( szHex, "0x%08X\0", *(piBsdDom2RlsReg) );
                  SetDlgItemText( hDlg, IDC_PI_DOM2RLS, szHex );

                  // RDRAM Interface (RI) Registers

                  wsprintf( szHex, "0x%08X\0", *(riModeReg) );
                  SetDlgItemText( hDlg, IDC_RI_MODE, szHex );
                  wsprintf( szHex, "0x%08X\0", *(riConfigReg) );
                  SetDlgItemText( hDlg, IDC_RI_CONFIG, szHex );
                  wsprintf( szHex, "0x%08X\0", *(riCurrentLoadReg) );
                  SetDlgItemText( hDlg, IDC_RI_CURLOAD, szHex );
                  wsprintf( szHex, "0x%08X\0", *(riSelectReg) );
                  SetDlgItemText( hDlg, IDC_RI_SELECT, szHex );
                  wsprintf( szHex, "0x%08X\0", *(riRefreshReg) );
                  SetDlgItemText( hDlg, IDC_RI_REFRESH, szHex );
                  wsprintf( szHex, "0x%08X\0", *(riLatencyReg) );
                  SetDlgItemText( hDlg, IDC_RI_LATENCY, szHex );
                  wsprintf( szHex, "0x%08X\0", *(riRerrorReg) );
                  SetDlgItemText( hDlg, IDC_RI_READERR, szHex );
                  wsprintf( szHex, "0x%08X\0", *(riWerrorReg) );
                  SetDlgItemText( hDlg, IDC_RI_WRITEERR, szHex );

                  return( TRUE );

                  break;
            }

            break;

         case WM_NOTIFY:

            switch( ((NMHDR FAR *)lParam)->code )
            {
               case PSN_SETACTIVE:

                  SendMessage( hDlg, WM_COMMAND, ID_REFRESH, 0L );

                  break;

               case PSN_APPLY:

                  SetWindowLong( hDlg, DWL_MSGRESULT, TRUE );

                  break;

               case PSN_KILLACTIVE:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );

                  return( TRUE );

                  break;

               case PSN_RESET:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );

                  break;

               default:

                  break;
            }

            break;
      }

      return( FALSE );
   }

