////////////////////////////////////////////////////////////////////////////////
// Filename  : debug.h
// File Desc : Integrated Debugger
// Project   : NSFE (Nintendo Sixty Four Emulator)
// Release   : 0.2a (Developer Release)
// Copyright : NSFE is the Copyright (c) 1999, Brian Faber.  
//
// This software is a Windows implementation of TrueReality, Copyright (C) 1999,
// Niki W. Waibel, and is published under the terms of the GNU General Public 
// License, see below:
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and or modify it under
// the terms of the GNU General Public Licence as published by the Free Software
// Foundation; either version 2 of the Licence, or any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public Licence for more details
//
// You should have received a copy of the GNU General Public Licence along with 
// this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
// Ave, Cambridge, MA 02139, USA.
//
// NSFE Author Information:
//    Brian Faber, Chino, Califorina - USA
//    bevis590@gte.net
//
//    Ben Marsh, Townsville, Queensland - Australia
//    photi@ozemail.com.au
//
// Original TrueReality Author Information:
//    Niki W. Waibel, Reichenau 20, 6890 Lustenau, Austria - EUROPE
//    niki.waibel@xms.net
// -----------------------------------------------------------------------------
// Update History
//
// 00001 16-02-1998 "Initial Code Release.                                     "

#ifndef _DEBUG_H_
#define _DEBUG_H_


////////////////////////////////////////////////////////////////////////////////
// Include Files

#ifndef _STDSDK_H_
#include "stdsdk.h"                    // Standard Win32 API Includes, etc.
#endif // _STDSDK_H_

#ifndef _TYPEDEFS_H_
#include "typedefs.h"                  // Type Definitions
#endif // _TYPEDEFS_H_


////////////////////////////////////////////////////////////////////////////////
// Prototypes

   int RegisterDebugWindow( void );
   BOOL APIENTRY DebugGPRegs( HWND, UINT, UINT, LONG );
   BOOL APIENTRY CP0Regs( HWND, UINT, UINT, LONG );
   BOOL APIENTRY CP1Regs( HWND, UINT, UINT, LONG );
   BOOL APIENTRY CP2RegsCore( HWND, UINT, UINT, LONG );
   BOOL APIENTRY CP2RegsVector( HWND, UINT, UINT, LONG );
   BOOL APIENTRY ViAiDpsSiRegs( HWND, UINT, UINT, LONG );
   BOOL APIENTRY PiRiRegs( HWND, UINT, UINT, LONG );


////////////////////////////////////////////////////////////////////////////////
// Globals

   HWND hwndDebugGPRegs;               // Dialog Handles
   HWND hwndCP0Regs;
   HWND hwndCP1Regs;
   HWND hwndCP2RegsCore;
   HWND hwndCP2RegsVector;
   HWND hwndViAiDpsSiRegs;
   HWND hwndPiRiRegs;

   char szHex[MAX_PATH];               // Hex Value String

   extern s64 gpReg[32];               // General Purpose Registers (GPRs)
   extern u64 cp0Reg[32];              // CP0 Registers
   extern u64 cp1Reg[32];              // Floating Point Registers
   extern u32 FCR0;                    // Implementation/Revision Register
   extern u32 FCR31;                   // Control/Status Register
   extern s64 cp2Reg[32];              // General Purpose Registers (RCP)
   extern VECTOR vReg[32];             // Vector Registers
   extern u32 PC;                      // Program Counter (PC)
   extern u64 HI;                      // HI Register
   extern u64 LO;                      // LO Register
   extern u8 LLBit;                    // Load/Link LLBit Register
   extern u32 instrWord;               // Instruction Word

   extern u32 *spMemAddrReg;           // SP COP0 Registers
   extern u32 *spDramAddrReg;        
   extern u32 *spRdLenReg;
   extern u32 *spWrLenReg;
   extern u32 *spStatusReg;
   extern u32 *spDmaFullReg;
   extern u32 *spDmaBusyReg;
   extern u32 *spSemaphoreReg;

   extern u32 *dpcStartReg;            // DP Command Registers
   extern u32 *dpcEndReg;
   extern u32 *dpcCurrentReg;
   extern u32 *dpcStatusReg;
   extern u32 *dpcClockReg;
   extern u32 *dpcBufBusyReg;
   extern u32 *dpcPipeBusyReg;
   extern u32 *dpcTmemReg;

   extern u32 *dpsTbistReg;          // DP Span Registers
   extern u32 *dpsTestModeReg;
   extern u32 *dpsBufTestAddrReg;
   extern u32 *dpsBufTestDataReg;

   extern u32 *viStatusReg;            // Video Interface (VI) Registers
   extern u32 *viOriginReg;
   extern u32 *viWidthReg;
   extern u32 *viIntrReg;
   extern u32 *viCurrentReg;
   extern u32 *viBurstReg;
   extern u32 *viVSyncReg;
   extern u32 *viHSyncReg;
   extern u32 *viLeapReg;
   extern u32 *viHStartReg;
   extern u32 *viVStartReg;
   extern u32 *viVBurstReg;
   extern u32 *viXScaleReg;
   extern u32 *viYScaleReg;

   extern u32 *aiDramAddrReg;        // Audio Interface (AI) Registers
   extern u32 *aiLenReg;
   extern u32 *aiControlReg;
   extern u32 *aiStatusReg;
   extern u32 *aiDacRateReg;
   extern u32 *aiBitRateReg;

   extern u32 *miInitModeReg;          // MIPS Interface (MI) Registers
   extern u32 *miVersionReg;         
   extern u32 *miIntrReg;
   extern u32 *miIntrMaskReg;

   extern u32 *piDramAddrReg;          // Peripheral Interface (PI) Registers
   extern u32 *piCartAddrReg;
   extern u32 *piRdLenReg;
   extern u32 *piWrLenReg;
   extern u32 *piStatusReg;
   extern u32 *piBsdDom1LatReg;
   extern u32 *piBsdDom1PwdReg;
   extern u32 *piBsdDom1PgsReg;
   extern u32 *piBsdDom1RlsReg;
   extern u32 *piBsdDom2LatReg;
   extern u32 *piBsdDom2PwdReg;
   extern u32 *piBsdDom2PgsReg;
   extern u32 *piBsdDom2RlsReg;

   extern u32 *riModeReg;              // RDRAM Interface (RI) Registers
   extern u32 *riConfigReg;
   extern u32 *riCurrentLoadReg;
   extern u32 *riSelectReg;
   extern u32 *riRefreshReg;
   extern u32 *riLatencyReg;
   extern u32 *riRerrorReg;
   extern u32 *riWerrorReg;

   extern u32 *siDramAddrReg;        // Serial Interface (SI) Registers
   extern u32 *siPifAddrRd64bReg;
   extern u32 *siPifAddrWr64bReg;
   extern u32 *siStatusReg;

   extern HANDLE hInst;                // Global Application Instance


#endif // _DEBUG_H_