////////////////////////////////////////////////////////////////////////////////
// Filename  : disassem.c
// File Desc : Integrated Debugger
// Project   : NSFE (Nintendo Sixty Four Emulator)
// Release   : 0.2a (Developer Release)
// Copyright : NSFE is the Copyright (c) 1999, Brian Faber.  
//
// This software is a Windows implementation of TrueReality, Copyright (C) 1999,
// Niki W. Waibel, and is published under the terms of the GNU General Public 
// License, see below:
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and or modify it under
// the terms of the GNU General Public Licence as published by the Free Software
// Foundation; either version 2 of the Licence, or any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public Licence for more details
//
// You should have received a copy of the GNU General Public Licence along with 
// this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
// Ave, Cambridge, MA 02139, USA.
//
// NSFE Author Information:
//    Brian Faber, Chino, Califorina - USA
//    bevis590@gte.net
//
//    Ben Marsh, Townsville, Queensland - Australia
//    photi@ozemail.com.au
//
// Original TrueReality Author Information:
//    Niki W. Waibel, Reichenau 20, 6890 Lustenau, Austria - EUROPE
//    niki.waibel@xms.net
// -----------------------------------------------------------------------------
// Update History
//
// 00001 16-02-1998 "Initial Code Release.                                     "

#ifndef _DISASSEM_H_
#include "disassem.h"
#endif // _DISASSEM_H_


////////////////////////////////////////////////////////////////////////////////
// Define the Property Sheet Pages for the Main Debug Window

   int MainDebugWindow( void )
   {
      PROPSHEETPAGE   psp[2];
      PROPSHEETHEADER psh;

      // Define the Disassembler Debug Page

      psp[0].dwSize      = sizeof( PROPSHEETPAGE );
      psp[0].dwFlags     = PSP_USETITLE;
      psp[0].hInstance   = hInst;
      psp[0].pszTemplate = MAKEINTRESOURCE( IDD_DISASSEMBLY );
      psp[0].pszIcon     = NULL;
      psp[0].pfnDlgProc  = (DLGPROC)Disassembler;
      psp[0].pszTitle    = "Disassembly";
      psp[0].lParam      = 0;

      // Define the Memory Viewer Debug Page

      psp[1].dwSize      = sizeof( PROPSHEETPAGE );
      psp[1].dwFlags     = PSP_USETITLE;
      psp[1].hInstance   = hInst;
      psp[1].pszTemplate = MAKEINTRESOURCE( IDD_MEMVIEW );
      psp[1].pszIcon     = NULL;
      psp[1].pfnDlgProc  = (DLGPROC)MemoryViewer;
      psp[1].pszTitle    = "Memory Viewer";
      psp[1].lParam      = 0;

      // Define Property Sheet Header

      psh.dwSize     = sizeof( PROPSHEETHEADER );
      psh.dwFlags    = PSH_PROPSHEETPAGE | PSH_NOAPPLYNOW | 
                       PSH_USEICONID | PSH_MODELESS;
      psh.hwndParent = NULL;
      psh.hInstance  = hInst;
      psh.pszIcon    = MAKEINTRESOURCE( IDI_DEBUG );
      psh.pszCaption = (LPSTR)"Debugger";
      psh.nPages     = sizeof( psp ) / sizeof( PROPSHEETPAGE );;
      psh.nStartPage = 0;
      psh.ppsp       = (LPCPROPSHEETPAGE) &psp;

      return( PropertySheet( &psh ) );
   }


////////////////////////////////////////////////////////////////////////////////
// Disassembler - Tabbed Dialog

   BOOL APIENTRY Disassembler( HWND hDlg, UINT message, 
                               UINT wParam, LONG lParam )
   {
      static PROPSHEETPAGE *ps;        // Pointer to Property Sheet Information

      switch( message )
      {
         case WM_INITDIALOG:

            // Save the PROPSHEETPAGE Information

            ps = (PROPSHEETPAGE *)lParam;
            hwndDisassembly = hDlg;

            return( TRUE );

            break;

		 case WM_KEYDOWN:

            switch( wParam )
            {
               case VK_RETURN:

                  SendMessage( hDlg, WM_COMMAND, IDC_DBG_STEP, 0L );

                  break;
            }
            
            break;

         case WM_COMMAND:

            switch( LOWORD( wParam ) )
            {
               case ID_REFRESH:

                  RefreshDisassembly( hDlg );

                  break;

               case IDC_DBG_STEP:

                  ResumeThread( hMainThread );

                  return( TRUE );

			   case IDC_DBG_SKIP:

				   PC = PC + 4;

				   RefreshDisassembly( hDlg );

				   break;

			   case IDC_DBG_GO:

				   // takes indebugger off so the thread does not supend

				   inDebugger = FALSE;

				   // Resumes the Emu Thread

				   ResumeThread( hMainThread );

				   break;

			   case IDC_DBG_BREAK:

				   // Suspends Thread

				   SuspendThread( hMainThread );

				   // Turns on so the step button works

				   inDebugger = TRUE;

				   // Refreshes debuger after break

				   RefreshDisassembly( hDlg );

				   break;
            }

            break;

         case WM_NOTIFY:

            switch( ((NMHDR FAR *)lParam)->code )
            {
               case PSN_SETACTIVE:

                  SendMessage( hDlg, WM_COMMAND, ID_REFRESH, 0L );

                  break;

               case PSN_APPLY:

                  SetWindowLong( hDlg, DWL_MSGRESULT, TRUE );

                  break;

               case PSN_KILLACTIVE:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );

                  return( TRUE );

                  break;

               case PSN_RESET:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );

                  break;

               default:

                  break;
            }

            break;
      }

      return( FALSE );
   }

   
////////////////////////////////////////////////////////////////////////////////
// Memory Viewer - Tabbed Dialog

   BOOL APIENTRY MemoryViewer( HWND hDlg, UINT message, 
                               UINT wParam, LONG lParam )
   {
      static PROPSHEETPAGE *ps;        // Pointer to Property Sheet Information

      switch( message )
      {
         case WM_INITDIALOG:

            // Save the PROPSHEETPAGE Information

            ps = (PROPSHEETPAGE *)lParam;

            return( TRUE );

            break;

         case WM_COMMAND:

            switch( LOWORD( wParam ) )
            {
               case ID_REFRESH:

                  return( TRUE );

                  break;
            }

            break;

         case WM_NOTIFY:

            switch( ((NMHDR FAR *)lParam)->code )
            {
               case PSN_SETACTIVE:

                  SendMessage( hDlg, WM_COMMAND, ID_REFRESH, 0L );

                  break;

               case PSN_APPLY:

                  SetWindowLong( hDlg, DWL_MSGRESULT, TRUE );

                  break;

               case PSN_KILLACTIVE:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );

                  return( TRUE );

                  break;

               case PSN_RESET:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );

                  break;

               default:

                  break;
            }

            break;
      }

      return( FALSE );
   }


////////////////////////////////////////////////////////////////////////////////
// Refresh Disassembly View

   BOOL RefreshDisassembly( HWND hDlg )
   {
      u32 i;                           // Loop Iterator
      u32 startPC;                     // Starting PC
      HWND hwndList;                   // Handle to the List Box
      u32 dbgInstr;                    // Instruction Word
      char disaL[80];                  // Disassembly Line
      char *params;                    // Opcode Parameter Order

      // Instruction Word Sub-divisions

      u8 dOpCode, drs, drt, drd, dsa, dFunct;
      s16 dImm;

      FillMemory( disasm, 2320, '\0' );

      startPC = PC - (14 * 4);

      for( i = 0; i < 29; i++ )
      {
         GetInstrWord((startPC + (i*4)));

         // Get Main Instruction Word Subdivisions
         // --------------------------------------
         // Bits [15:0]    Imm
         // Bits [20:16]   rt
         // Bits [25:21]   rs
         // Bits [31:26]   OpCode
         // Bits [5:0]     Funct
         // Bits [10:6]    sa
         // Bits [15:11]   rd
         
         __asm {
            mov   eax,[dbgInstr]    // Get Immediate Value
            mov   [dImm],ax          
            mov   ecx,eax           // Get OpCode        
            shr   ecx,1Ah
            mov   [dOpCode],cl
            mov   edx,eax           // Get 'rs' Value
            shr   edx,15h
            and   edx,1Fh
            mov   [drs],dl
            shr   eax,10h           // Get 'rt' Value
            and   eax,1Fh
            mov   [drt],al
            mov   eax,[dbgInstr]    // Get 'rd' Value
            mov   ecx,eax
            shr   ecx,0Bh
            and   ecx,1Fh
            mov   [drd],cl
            mov   edx,eax           // Get 'sa' Value
            shr   edx,06h
            and   edx,1Fh
            mov   [dsa],dl
            and   eax,3Fh           // Get 'Funct' Value
            mov   [dFunct],al
         };

         // Put Program Counter and Intruction Word in Disassembly

         wsprintf( disaL, "    %08X   <%08X>     ",
                  (startPC + (i*4)), dbgInstr );

         if( dbgInstr == 0x00000000 )
         {
            wsprintf( disaL + strlen(disaL), "NOP," );

            // Properly Terminate the String

            disaL[ (strlen(disaL) - 1) ] = '\0';

            // Copy the Created String into the Disassembly Table

            CopyMemory( disasm[i], disaL, strlen(disaL) );

            continue;
         }

         // Add Opcode to Disassembly and Get Opcode Parameter Order

         if( dOpCode == 0 )
         {
            wsprintf( disaL + strlen(disaL), "%-8s ", SpecialName[dFunct] );
            params = SpecialParams[dFunct];
         }
         else if( dOpCode == 1 )
         {
            wsprintf( disaL + strlen(disaL), "%-8s ", RegimmName[drt] );
            params = RegimmParams[drt];
         }
         else
         {
            if( dOpCode == 16 )
            {
               wsprintf( disaL + strlen(disaL), "%-8s ", COP0Name[drs] );
               params = COP0Params[drs];
            }
            else
            {
               wsprintf( disaL + strlen(disaL), "%-8s ", OpcodeName[dOpCode] );
               params = OpcodeParams[dOpCode];
            }
         }

         // Add Parameters to Disassembly

         while( *params != '\0' )
         {
            switch( *params )
            {
               case 'a':   // sa

                  wsprintf( disaL + strlen(disaL), "0x%02X,", dsa );

                  break;

               case 's':   // GPReg (rs)

                  wsprintf( disaL + strlen(disaL), "%2s,",
                            GPRegName[drs] );
                  break;

               case 't':   // GPReg (rt)

                  wsprintf( disaL + strlen(disaL), "%2s,",
                            GPRegName[drt] );
                  break;

               case 'd':   // GPReg (rd)

                  wsprintf( disaL + strlen(disaL), "%2s,",
                            GPRegName[drd] );
                  break;

               case 'y':   // COP0 (rd)

                  wsprintf( disaL + strlen(disaL), "%-9s,",
                            COP0RegName[drd] );
                  break;

               case 'i':   // Immediate

                  wsprintf( disaL + strlen(disaL), "0x%04X,", (u16)dImm );

                  break;

               case 'o':   // offset(base)

                  wsprintf( disaL + strlen(disaL), "0x%04X(%2s),", 
                           (u16)dImm, GPRegName[drs] );

                  break;

               case 'b':   // Branch Offset

                  wsprintf( disaL + strlen(disaL), "0x%08X,",
                            ((startPC + (i*4)) + 4) + (((s32)(dImm)) << 2) );

                  break;

               case 'z':   // Not Yet Implemented

                  wsprintf( disaL + strlen(disaL), "Not Yet Implemented," );

                  break;

               case 'g':   // Jump Target

                  wsprintf( disaL + strlen(disaL), "0x%08X,",
                            (((startPC + (i*4)) + 4) & 0xF0000000) |
                            ((dbgInstr & 0x03FFFFFF) << 2) );

                  break;
            }

            params++;

         }

         // Properly Terminate the String

         disaL[ (strlen(disaL) - 1) ] = '\0';

         // Copy the Created String into the Disassembly Table

         CopyMemory( disasm[i], disaL, strlen(disaL) );
      }

      // Get Handle to List Box Control

      hwndList = GetDlgItem( hDlg, IDC_DISALIST );  

      // Clear down List Box Control

      SendMessage( hwndList, LB_RESETCONTENT, 0, 0 );

      // Fill List Box Control with Refreshed Information

      for( i = 0; i < 29; i++ )
      {
         SendMessage( hwndList, LB_ADDSTRING, 0, (LPARAM)disasm[i] );
      }

      // Highlight Current Line of Disassembly

      SendMessage( hwndList, LB_SELECTSTRING, 13, (LPARAM)disasm[14] );

      return( TRUE );
   }
