////////////////////////////////////////////////////////////////////////////////
// Filename  : disassem.h
// File Desc : Integrated Debugger
// Project   : NSFE (Nintendo Sixty Four Emulator)
// Release   : 0.2a (Developer Release)
// Copyright : NSFE is the Copyright (c) 1999, Brian Faber.  
//
// This software is a Windows implementation of TrueReality, Copyright (C) 1999,
// Niki W. Waibel, and is published under the terms of the GNU General Public 
// License, see below:
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and or modify it under
// the terms of the GNU General Public Licence as published by the Free Software
// Foundation; either version 2 of the Licence, or any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public Licence for more details
//
// You should have received a copy of the GNU General Public Licence along with 
// this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
// Ave, Cambridge, MA 02139, USA.
//
// NSFE Author Information:
//    Brian Faber, Chino, Califorina - USA
//    bevis590@gte.net
//
//    Ben Marsh, Townsville, Queensland - Australia
//    photi@ozemail.com.au
//
// Original TrueReality Author Information:
//    Niki W. Waibel, Reichenau 20, 6890 Lustenau, Austria - EUROPE
//    niki.waibel@xms.net
// -----------------------------------------------------------------------------
// Update History
//
// 00001 16-02-1998 "Initial Code Release.                                     "

#ifndef _DISASSEM_H_
#define _DISASSEM_H_


////////////////////////////////////////////////////////////////////////////////
// Include Files

#ifndef _STDSDK_H_
#include "stdsdk.h"                    // Standard Win32 API Includes, etc.
#endif // _STDSDK_H_

#ifndef _TYPEDEFS_H_
#include "typedefs.h"                  // Type Definitions
#endif // _TYPEDEFS_H_

#ifndef _R4300_H_
#include "r4300.h"                     // Core R4300 Definitions
#endif // _R4300_H_


////////////////////////////////////////////////////////////////////////////////
// Prototypes

   int MainDebugWindow( void );
   BOOL APIENTRY Disassembler( HWND, UINT, UINT, LONG );
   BOOL APIENTRY MemoryViewer( HWND, UINT, UINT, LONG );
   BOOL RefreshDisassembly( HWND );

   extern u32 *MemoryAccess( u32, u32 );


////////////////////////////////////////////////////////////////////////////////
// Globals

   // General Purpose Register Names

   char *GPRegName[32] =
   {
      "R0", "AT", "V0", "V1", "A0", "A1", "A2", "A3",
      "T0", "T1", "T2", "T3", "T4", "T5", "T6", "T7",
      "S0", "S1", "S2", "S3", "S4", "S5", "S6", "S7",
      "T8", "T9", "K0", "K1", "GP", "SP", "S8", "RA"
   };

   // COP0 Register Names

   char *COP0RegName[32] =
   {
      "Index",       "Random",      "EntryLo0",    "EntryLo1", 
      "Context",     "PageMask",    "Wired",       "RFU",
      "BadVAddr",    "Count",       "EntryHi",     "Compare", 
      "Status",      "Cause",       "EPC",         "PRId",
      "Config",      "LLAddr",      "WatchLo",     "WatchHi", 
      "XContext",    "RFU",         "RFU",         "RFU",
      "RFU",         "RFU",         "ParityErr",   "CacheErr", 
      "TagLo",       "TagHi",       "ErrorEPC",    "RFU"
   };

   // Vector Register Names

   char *VRegName[32] =
   {
      "$V0",  "$V1",  "$V2",  "$V3",  "$V4",  "$V5",  "$V6",  "$V7",
      "$V8",  "$V9",  "$V10", "$V11", "$V12", "$V13", "$V14", "$V15",
      "$V16", "$V17", "$V18", "$V19", "$V20", "$V21", "$V22", "$V23",
      "$V24", "$V25", "$V26", "$V27", "$V28", "$V29", "$V30", "$V31"
   };

   // Opcode Name and Parameter Table

   char *OpcodeName[64] =
   {
      "NOP",          "???",          "J",            "JAL",
      "BEQ",          "BNE",          "BLEZ",         "BGTZ",
      "ADDI",         "ADDIU",        "SLTI",         "SLTIU",
      "ANDI",         "ORI",          "XORI",         "LUI",
      "COP0",         "COP1",         "COP2",         "???",
      "BEQL",         "BNEL",         "BLEZL",        "BGTZL",
      "DADDI",        "DADDIU",       "LDL",          "LDR",
      "???",          "???",          "???",          "???",
      "LB",           "LH",           "LWL",          "LW",
      "LBU",          "LHU",          "LWR",          "LWU",
      "SB",           "SH",           "SWL",          "SW",
      "SDL",          "SDR",          "SWR",          "CACHE",
      "LL",           "LWC1",         "LWC2",         "???",
      "LLD",          "LDC1",         "LDC2",         "LD",
      "SC",           "SWC1",         "SWC2",         "???",
      "SCD",          "SDC1",         "SDC2",         "SD" 
   };

   char *OpcodeParams[64] =
   {
      "",             "",             "g",            "g",
      "stb",          "stb",          "sb",           "sb",
      "tsi",          "tsi",          "tsi",          "tsi",
      "tsi",          "tsi",          "tsi",          "ti",
      "",             "",             "",             "",
      "stb",          "stb",          "sb",           "sb",
      "tsi",          "tsi",          "to",           "to",
      "",             "",             "",             "",     
      "to",           "to",           "to",           "to",
      "to",           "to",           "to",           "to",
      "to",           "to",           "to",           "to",
      "to",           "to",           "to",           "to",
      "to",           "to",           "to",           "",
      "to",           "to",           "to",           "to",
      "to",           "to",           "to",           "",
      "to",           "to",           "to",           "to"     
   };

   // Special Opcode Name and Parameter tables

   char *SpecialName[64] =
   {
      "SLL",          "???",          "SRL",          "SRA",
      "SLLV",         "???",          "SRLV",         "SRAV",
      "JR",           "JALR",         "???",          "???",
      "SYSCALL",      "BREAK",        "???",          "SYNC",
      "MFHI",         "MTHI",         "MFLO",         "MTLO",
      "DSLLV",        "???",          "DSRLV",        "DSRAV",
      "MULT",         "MULTU",        "DIV",          "DIVU",
      "DMULT",        "DMULTU",       "DDIV",         "DDIVU",
      "ADD",          "ADDU",         "SUB",          "SUBU",
      "AND",          "OR",           "XOR",          "NOR",
      "???",          "???",          "SLT",          "SLTU",
      "DADD",         "DADDU",        "DSUB",         "DSUBU",
      "TGE",          "TGEU",         "TLT",          "TLTU",
      "TEQ",          "???",          "TNE",          "???",
      "DSLL",         "???",          "DSRL",         "DSRA",
      "DSLL32",       "???",          "DSRL32",       "DSRA32"   
   };

   char *SpecialParams[64] = 
   {
      "dta",          "",             "dta",          "dta",
      "dts",          "",             "dts",          "dts",
      "s",            "s",            "",             "",
      "",             "",             "",             "",
      "d",            "s",            "d",            "s",
      "dts",          "",             "dts",          "dts",
      "st",           "st",           "st",           "st",
      "st",           "st",           "st",           "st",     
      "dst",          "dst",          "dst",          "dst",
      "dst",          "dst",          "dst",          "dst",
      "",             "",             "dst",          "dst",
      "dst",          "dst",          "dst",          "dst",
      "z",            "z",            "z",            "z",
      "z",            "",             "z",            "",
      "dta",          "",             "dta",          "dta",
      "dta",          "",             "dta",          "dta"     
   };

   // Register Immediate Opcode Name and Parameter Tables

   char *RegimmName[32] =
   {
      "BLTZ",         "BGEZ",         "BLTZL",        "BGEZL",
      "???",          "???",          "???",          "???",
      "TGEI",         "TGEIU",        "TLTI",         "TLTIU",
      "TEQI",         "TNEI",         "???",          "???",
      "BLTZAL",       "BGEZAL",       "BLTZALL",      "BGEZALL",
      "???",          "???",          "???",          "???",
      "???",          "???",          "???",          "???",
      "???",          "???",          "???",          "???"    
   };

   char *RegimmParams[32] = 
   {
      "sb",           "sb",        "sb",        "sb",
      "",             "",          "",          "",
      "z",            "z",         "z",         "z",
      "z",            "z",         "",          "",
      "sb",           "sb",        "sb",        "sb",
      "",             "",          "",          "",
      "",             "",          "",          "",
      "",             "",          "",          ""     
   };

   // COP1 Opcode Name and Parameter Tables

   char *COP0Name[32] =
   {
      "???",          "???",       "???",       "???",
      "MTC0",         "???",       "???",       "???",
      "???",          "???",       "???",       "???",
      "???",          "???",       "???",       "???",
      "???",          "???",       "???",       "???",
      "???",          "???",       "???",       "???",
      "???",          "???",       "???",       "???",
      "???",          "???",       "???",       "???"     
   };

   char *COP0Params[32] =
   {
      "",             "",          "",          "",
      "ty",           "",          "",          "",
      "",             "",          "",          "",
      "",             "",          "",          "",
      "",             "",          "",          "",
      "",             "",          "",          "",
      "",             "",          "",          "",
      "",             "",          "",          ""     
   };

   u32 bPoint[50];                     // Breakpoint Addresses
   u32 bPI = 0;                        // Breakpoint Index

   char disasm[29][80];                // Disassembly Lines
   BOOL inDebugger = FALSE;            // Is the Debugger Running
   HWND hwndDisassembly;               // Handle to Disasm Window

   extern HANDLE hMainThread;          // Handle to the Main N64 Thread
   extern HANDLE hInst;                // Global Application Instance
   extern u32 PC;                      // Program Counter (PC)
   extern u32 instrWord;               // Instruction Word
   extern u32 *bootCode;               // Boot Code
   extern u32 *gameCode;               // Game Code
   extern u32 *rdramMem;               // RDRAM Memory
   extern pRomImageHeader ROM;         // Rom Header Information
   extern u32 cartridgeSize;           // ROM Cartridge Size (in Bytes)
   extern DWORD ExecNintendo64( LPVOID ); 
   extern DWORD n64Tid;                // Main N64 Thread ID
   extern HANDLE hMainThread;          // Handle to the Main N64 Thread
   extern HWND hwndMain;               // Handle to the Main App Window
   extern BOOL inDebugger;             // Is the Debugger Running



   // Code for Getting the Instruction Word

   #define GetInstrWord(x)                                         \
      if((x >= 0x00000000) && (x < 0x0040FFFF))                    \
         dbgInstr = *(rdramMem + (x / 4));                         \
      else                                                         \
         if((x >= ROM->gameAddress) &&                             \
            (x < (ROM->gameAddress + cartridgeSize)))              \
            dbgInstr = *(gameCode + ((x - ROM->gameAddress) / 4)); \
         else                                                      \
            if((x >= 0xA4000040) && (x < 0xA4001000))              \
               dbgInstr = *(bootCode + ((x - 0xA4000040) / 4));    \
            else                                                   \
               dbgInstr = *(MemoryAccess(x, MEMACCESS_READ));


#endif // _DEBUG_H_