////////////////////////////////////////////////////////////////////////////////
// Filename  : interupt.h
// File Desc : Interrupt Handling
// Project   : NSFE (Nintendo Sixty Four Emulator)
// Release   : 0.2a (Developer Release)
// Copyright : NSFE is the Copyright (c) 1999, Brian Faber.  
//
// This software is a Windows implementation of TrueReality, Copyright (C) 1999,
// Niki W. Waibel, and is published under the terms of the GNU General Public 
// License, see below:
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and or modify it under
// the terms of the GNU General Public Licence as published by the Free Software
// Foundation; either version 2 of the Licence, or any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public Licence for more details
//
// You should have received a copy of the GNU General Public Licence along with 
// this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
// Ave, Cambridge, MA 02139, USA.
//
// NSFE Author Information:
//    Brian Faber, Chino, Califorina - USA
//    bevis590@gte.net
//
//    Ben Marsh, Townsville, Queensland - Australia
//    photi@ozemail.com.au
//
// Original TrueReality Author Information:
//    Niki W. Waibel, Reichenau 20, 6890 Lustenau, Austria - EUROPE
//    niki.waibel@xms.net
// -----------------------------------------------------------------------------
// Update History
//
// 00001 16-02-1998 "Initial Code Release.                                     "

#ifndef _INTERUPT_H_
#define _INTERUPT_H_


////////////////////////////////////////////////////////////////////////////////
// Include Files

#ifndef _STDSDK_H_
#include "stdsdk.h"                    // Standard Win32 API Includes, etc.
#endif // _STDSDK_H_

#ifndef _TYPEDEFS_H_
#include "typedefs.h"                  // Type Definitions
#endif // _TYPEDEFS_H_

#ifndef _R4300_H_
#include "r4300.h"                     // Core R4300 Definitions
#endif // _R4300_H_

#ifndef _RCPMEM_H_
#include "rcpmem.h"                    // Register and Bit Defs for the RCP Mem
#endif // _RCPMEM_H_


////////////////////////////////////////////////////////////////////////////////
// Prototypes

   void InitInterrupts( void );
   void CheckMIInterrupts( void );
   void CheckCPUInterrupts( void );
   void CompareInterrupt( void );
   void MIInterruptVI( void );
   void MIInterruptSI( void );
   static void CommonExceptionVector( void );


////////////////////////////////////////////////////////////////////////////////
// Globals

   INTERRUPTS Interrupts[8];           // Interrupt Table
   static u32 miOldIntrMaskReg = MI_INTR_NO;

   extern u64 cp0Reg[];                // Coprocessor 0 Registers
   extern u32 PC;                      // Program Counter (PC)
   extern u32 branchDelay;             // Branch Delay Status

   extern u32 *miIntrReg;              // MI_INTR_REG
   extern u32 *miIntrMaskReg;          // MI_INTR_MASK_REG


#endif // _INTERUPT_H_
