////////////////////////////////////////////////////////////////////////////////
// Filename  : memory.h
// File Desc : Emulated Memory
// Project   : NSFE (Nintendo Sixty Four Emulator)
// Release   : 0.2a (Developer Release)
// Copyright : NSFE is the Copyright (c) 1999, Brian Faber.  
//
// This software is a Windows implementation of TrueReality, Copyright (C) 1999,
// Niki W. Waibel, and is published under the terms of the GNU General Public 
// License, see below:
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and or modify it under
// the terms of the GNU General Public Licence as published by the Free Software
// Foundation; either version 2 of the Licence, or any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public Licence for more details
//
// You should have received a copy of the GNU General Public Licence along with 
// this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
// Ave, Cambridge, MA 02139, USA.
//
// NSFE Author Information:
//    Brian Faber, Chino, Califorina - USA
//    bevis590@gte.net
//
//    Ben Marsh, Townsville, Queensland - Australia
//    photi@ozemail.com.au
//
// Original TrueReality Author Information:
//    Niki W. Waibel, Reichenau 20, 6890 Lustenau, Austria - EUROPE
//    niki.waibel@xms.net
// -----------------------------------------------------------------------------
// Update History
//
// 00001 16-02-1998 "Initial Code Release.                                     "

#ifndef _MEMORY_H_
#define _MEMORY_H_


////////////////////////////////////////////////////////////////////////////////
// Include Files

#ifndef _STDSDK_H_
#include "stdsdk.h"                    // Standard Win32 API Includes, etc.
#endif // _STDSDK_H_

#ifndef _TYPEDEFS_H_
#include "typedefs.h"                  // Type Definitions
#endif // _TYPEDEFS_H_

#ifndef _R4300_H_
#include "r4300.h"                     // Core R4300 Definitions
#endif // _R4300_H_

#ifndef _RCPMEM_H_
#include "rcpmem.h"                    // Reg and Bit Defs for the RCP Mem Map
#endif // _RCPMEM_H_


////////////////////////////////////////////////////////////////////////////////
// Macros

// Memory to be allocated for RCP Memory/Registers

#define  MEMSIZE              0x4D2930


////////////////////////////////////////////////////////////////////////////////
// Globals

   // Memory Block Pointers

   u32 *romImage;             // Header, Boot and Game Code
   u32 *bootCode;             // Boot Code (0xA4000040 - 0xA0000FFF)
   u32 *gameCode;             // Address Dependant on Header Information

   u32 *rcpMem;               // RCP Memory 

                              // Start        End         Size      Offset
                              // -------------------------------------------
   u32 *rdramMem;             // 0x00000000 - 0x0040FFFF  0x410000  0x000000
   u32 *rdramRegs;            // 0x03F00000 - 0x03F80027  0x080028  0x410000
   u32 *spMem;                // 0x04000000 - 0x04001FFF  0x002000  0x490028
   u32 *spRegs;               // 0x04040000 - 0x04080007  0x040008  0x492028
   u32 *dpCmdRegs;            // 0x04100000 - 0x0410001F  0x000020  0x4D2030
   u32 *dpSpanRegs;           // 0x04200000 - 0x0420000F  0x000010  0x4D2050
   u32 *miRegs;               // 0x04300000 - 0x0430000F  0x000010  0x4D2060
   u32 *viRegs;               // 0x04400000 - 0x04400037  0x000038  0x4D2070
   u32 *aiRegs;               // 0x04500000 - 0x04500017  0x000018  0x4D20A8
   u32 *piRegs;               // 0x04600000 - 0x04600033  0x000034  0x4D20C0
   u32 *riRegs;               // 0x04700000 - 0x0470001F  0x000020  0x4D20F4
   u32 *siRegs;               // 0x04800000 - 0x0480001B  0x00001C  0x4D2114
   u32 *pifRom;               // 0x1FC00000 - 0x1FC007BF  0x0007C0  0x4D2130
   u32 *pifRam;               // 0x1FC007C0 - 0x1FC007FF  0x000040  0x4D28F0

   // RCP Register/Memory Pointers

   u32 *spMemAddrReg;         // SP COP0 Registers
   u32 *spDramAddrReg;        
   u32 *spRdLenReg;
   u32 *spWrLenReg;
   u32 *spStatusReg;
   u32 *spDmaFullReg;
   u32 *spDmaBusyReg;
   u32 *spSemaphoreReg;
   u32 *spPc;

   u32 *dpcStartReg;          // DP Command Registers
   u32 *dpcEndReg;
   u32 *dpcCurrentReg;
   u32 *dpcStatusReg;
   u32 *dpcClockReg;
   u32 *dpcBufBusyReg;
   u32 *dpcPipeBusyReg;
   u32 *dpcTmemReg;

   u32 *dpsTbistReg;          // DP Span Registers
   u32 *dpsTestModeReg;
   u32 *dpsBufTestAddrReg;
   u32 *dpsBufTestDataReg;

   u32 *miInitModeReg;        // MIPS Interface (MI) Registers
   u32 *miVersionReg;         
   u32 *miIntrReg;
   u32 *miIntrMaskReg;

   u32 *viStatusReg;          // Video Interface (VI) Registers
   u32 *viOriginReg;
   u32 *viWidthReg;
   u32 *viIntrReg;
   u32 *viCurrentReg;
   u32 *viBurstReg;
   u32 *viVSyncReg;
   u32 *viHSyncReg;
   u32 *viLeapReg;
   u32 *viHStartReg;
   u32 *viVStartReg;
   u32 *viVBurstReg;
   u32 *viXScaleReg;
   u32 *viYScaleReg;

   u32 *aiDramAddrReg;        // Audio Interface (AI) Registers
   u32 *aiLenReg;
   u32 *aiControlReg;
   u32 *aiStatusReg;
   u32 *aiDacRateReg;
   u32 *aiBitRateReg;
   
   u32 *piDramAddrReg;        // Peripheral Interface (PI) Registers
   u32 *piCartAddrReg;
   u32 *piRdLenReg;
   u32 *piWrLenReg;
   u32 *piStatusReg;
   u32 *piBsdDom1LatReg;
   u32 *piBsdDom1PwdReg;
   u32 *piBsdDom1PgsReg;
   u32 *piBsdDom1RlsReg;
   u32 *piBsdDom2LatReg;
   u32 *piBsdDom2PwdReg;
   u32 *piBsdDom2PgsReg;
   u32 *piBsdDom2RlsReg;

   u32 *riModeReg;            // RDRAM Interface (RI) Registers
   u32 *riConfigReg;
   u32 *riCurrentLoadReg;
   u32 *riSelectReg;
   u32 *riRefreshReg;
   u32 *riLatencyReg;
   u32 *riRerrorReg;
   u32 *riWerrorReg;

   u32 *siDramAddrReg;        // Serial Interface (SI) Registers
   u32 *siPifAddrRd64bReg;
   u32 *siPifAddrWr64bReg;
   u32 *siStatusReg;

   u32 memAction = 0;                  // Memory Action Flags
   u32 prevViStatusReg = 0;            // Previous VI Status Reg Value
   u32 prevViOriginReg = 0;            // Previous VI Origin Reg Value
   u32 prevViWidthReg = 0;             // Previous VI Width Reg Value

   extern BOOL cpuIsRunning;           // Is Processor Executing
   extern u32 nullMemory = 0;          // Safe Storage of Unmapped Memory
   extern char szBuffer[];             // Temporary String Buffer
   extern u32 PC;                      // Program Counter
   extern u64 cp0Reg[];                // COP0 Registers
   extern HWND hwndMain;               // Handle to the Main App Window
   extern u32 cpuAction;               // CPU Action Flags
   extern u32 dmaFlags;                // DMA Identification Flags
   extern u32 cartridgeSize;           // ROM Cartridge Size (in Bytes)


////////////////////////////////////////////////////////////////////////////////
// Prototypes

   void FreeMem( void );
   BOOL InitMem( void );
   u32 *MemoryAccess( u32, u32 );
   void UpdateMIReg( void );
   void CheckVIReg( void );
   void UpdatePIReg( void );

#define PI_STATUS_CHANGE()                                              \
   *((s32 *)MemoryAccess(PI_STATUS_REG, MEMACCESS_WRITE)) = 0x00000000;


#endif // _MEMORY_H_
