////////////////////////////////////////////////////////////////////////////////
// Filename  : n64.h
// File Desc : Nintendo 64 Core Emulation
// Project   : NSFE (Nintendo Sixty Four Emulator)
// Release   : 0.2a (Developer Release)
// Copyright : NSFE is the Copyright (c) 1999, Brian Faber.  
//
// This software is a Windows implementation of TrueReality, Copyright (C) 1999,
// Niki W. Waibel, and is published under the terms of the GNU General Public 
// License, see below:
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and or modify it under
// the terms of the GNU General Public Licence as published by the Free Software
// Foundation; either version 2 of the Licence, or any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public Licence for more details
//
// You should have received a copy of the GNU General Public Licence along with 
// this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
// Ave, Cambridge, MA 02139, USA.
//
// NSFE Author Information:
//    Brian Faber, Chino, Califorina - USA
//    bevis590@gte.net
//
//    Ben Marsh, Townsville, Queensland - Australia
//    photi@ozemail.com.au
//
// Original TrueReality Author Information:
//    Niki W. Waibel, Reichenau 20, 6890 Lustenau, Austria - EUROPE
//    niki.waibel@xms.net
// -----------------------------------------------------------------------------
// Update History
//
// 00001 16-02-1998 "Initial Code Release.                                     "

#ifndef _N64_H_
#define _N64_H_


////////////////////////////////////////////////////////////////////////////////
// Module Include Files

#ifndef _STDSDK_H_
#include "stdsdk.h"                    // Standard Win32 API Includes, etc.
#endif // _STDSDK_H_

#ifndef _REGISTER_H_
#include "register.h"                  // Register Definitions
#endif // _REGISTER_H_

#ifndef _OPCODE_H_
#include "opcode.h"                    // OpCode Definitions
#endif // _OPCODE_H_

#ifndef _TYPEDEFS_H_
#include "typedefs.h"                  // Type Definitions
#endif // _TYPEDEFS_H_

#ifndef _R4300_H_
#include "r4300.h"                     // Core R4300 Definitions
#endif // _R4300_H_


////////////////////////////////////////////////////////////////////////////////
// Procedure Prototypes

   DWORD ExecNintendo64( LPVOID );
   void InitHardware( void );
   void ErrUnhandledOpCode( LPSTR );

   extern u32 *MemoryAccess( u32, u32 );
   extern void DMAHandler( void );
   extern void InitInterrupts( void );
   extern void CheckMIInterrupts( void );
   extern void CheckCPUInterrupts( void );
   extern void CompareInterrupt( void );
   extern void UpdateMIReg( void );
   extern void CheckVIReg( void );
   extern void UpdatePIReg( void );
   

////////////////////////////////////////////////////////////////////////////////
// Globals

   // Instruction Word

   u32 instrWord;

   // Instruction Word Sub-divisions

   u8 OpCode, rs, rt, rd, sa, Funct;
   s16 Imm;

   // Branch Delay Slot

   u32 branchPC;                    // Delayed Program Counter
   u32 branchDelay = 0;             // Branch Delay Status
                                    // 0 = Normal Execution
                                    // 1 = Branch Delay Instruction
                                    // 2 = PC becomes Branch PC
   
   u32 cpuAction = 0;               // CPU Action Flags

   BOOL cpuIsRunning = TRUE;        // Is Processor Executing

   extern HANDLE hMainThread;       // Handle to the Main N64 Thread
   extern BOOL inDebugger;          // Is the Debugger Running
   extern u32 memAction;            // Memory Action Flags
   extern INTERRUPTS Interrupts[];  // Interrupt Table
   extern DWORD dmaTid;             // Main N64 Thread ID
   extern HANDLE hDmaThread;        // Handle to the Main N64 Thread
   extern u32 baseAddress;          // Boot Code Base Address
   extern u32 cartridgeSize;        // ROM Cartridge Size (in Bytes)
   extern pRomImageHeader ROM;      // Rom Header Information
   extern char szBuffer[];          // Temporary String Buffer
   extern HWND hwndMain;            // Handle to Main App Window

   extern HWND hwndDebugGPRegs;     // Dialog Handles
   extern HWND hwndCP0Regs;
   extern HWND hwndCP1Regs;
   extern HWND hwndCP2RegsCore;
   extern HWND hwndCP2RegsVector;
   extern HWND hwndViAiDpsSiRegs;
   extern HWND hwndPiRiRegs;
   extern HWND hwndDisassembly;  

   extern u32 *rdramMem;            // RDRAM Memory
   extern u32 *bootCode;            // ROM Boot Code
   extern u32 *gameCode;            // ROM Game Code

   extern u32 *miVersionReg;        // MIPS Version Register
   extern u32 *piStatusReg;         // PI Status Register

   // Do Relevant Actions when Writing to Memory
   // For Speed Reasons, Code is Inline

   #define MemWriteCheck()                      \
      if( memAction )                           \
      {                                         \
         if( memAction & CHECK_VI_REG )         \
            CheckVIReg();                       \
         if( memAction & DMA_REQUEST_MASK )     \
            DMAHandler();                       \
         if( memAction & UPDATE_PI_REG )        \
            UpdatePIReg();                      \
         if( memAction & UPDATE_MI_REG )        \
            UpdateMIReg();                      \
      }                                         \
      memAction = CLEAR_ACTIONS;

   // Code for Getting the Instruction Word

   #define GetInstrWord()                                            \
      if((PC >= 0x00000000) && (PC < 0x0040FFFF))                    \
         instrWord = *(rdramMem + (PC / 4));                         \
      else                                                           \
         if((PC >= ROM->gameAddress) &&                              \
            (PC < (ROM->gameAddress + cartridgeSize)))               \
            instrWord = *(gameCode + ((PC - ROM->gameAddress) / 4)); \
         else                                                        \
            if((PC >= 0xA4000040) && (PC < 0xA4001000))              \
               instrWord = *(bootCode + ((PC - 0xA4000040) / 4));    \
            else                                                     \
               instrWord = *(MemoryAccess(PC, MEMACCESS_READ));


#endif // _N64_H_