////////////////////////////////////////////////////////////////////////////////
// Filename  : nsfe.c
// File Desc : Nintendo Sixty Four Emulator
// Project   : NSFE (Nintendo Sixty Four Emulator)
// Release   : 0.2a (Developer Release)
// Copyright : NSFE is the Copyright (c) 1999, Brian Faber.  
//
// This software is a Windows implementation of TrueReality, Copyright (C) 1999,
// Niki W. Waibel, and is published under the terms of the GNU General Public 
// License, see below:
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and or modify it under
// the terms of the GNU General Public Licence as published by the Free Software
// Foundation; either version 2 of the Licence, or any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public Licence for more details
//
// You should have received a copy of the GNU General Public Licence along with 
// this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
// Ave, Cambridge, MA 02139, USA.
//
// NSFE Author Information:
//    Brian Faber, Chino, Califorina - USA
//    bevis590@gte.net
//
//    Ben Marsh, Townsville, Queensland - Australia
//    photi@ozemail.com.au
//
// Original TrueReality Author Information:
//    Niki W. Waibel, Reichenau 20, 6890 Lustenau, Austria - EUROPE
//    niki.waibel@xms.net
// -----------------------------------------------------------------------------
// Update History
//
// 00001 16-02-1998 "Initial Code Release.                                     "

#ifndef _NSFE_H_
#include "nsfe.h"
#endif // _NSFE_H_


////////////////////////////////////////////////////////////////////////////////
// Main Application Code Starts Here

   int WINAPI WinMain( HINSTANCE hThisInst,  // Handle to this Program Instance
                       HINSTANCE hPrevInst,  // Previous Instance (Obsolete)
                       LPSTR lpszArgs,       // Passed Arguments
                       int nWinMode )        // Windows Display Mode
   {
      WNDCLASSEX wclex;                // Pointer to Windows Class
      HWND       hwnd;                 // Window Handle
      MSG        msg;                  // Windows Message Pointer

      // Define the Primary Windows Class

      wclex.hInstance     = hThisInst;
      wclex.lpszClassName = szClassName;
      wclex.lpfnWndProc   = WindowFunc;
      wclex.style         = 0;
      wclex.cbSize        = sizeof( WNDCLASSEX );
      wclex.hIcon         = LoadIcon( hThisInst, MAKEINTRESOURCE(IDI_N64LOGO) );
      wclex.hIconSm       = LoadIcon( hThisInst, MAKEINTRESOURCE(IDI_N64LOGO) );
      wclex.hCursor       = LoadCursor( NULL, IDC_ARROW );
      wclex.lpszMenuName  = MAKEINTRESOURCE( IDR_NSFEMENU );
      wclex.cbClsExtra    = 0;
      wclex.cbWndExtra    = 0;
      wclex.hbrBackground = (HBRUSH) GetStockObject( BLACK_BRUSH );

      // Register the Window Class

      if( !RegisterClassEx( &wclex ) )
         return( 0 );

      // Create the Main Application Window

      hwnd = CreateWindow( szClassName, 
                           "NSFE \"Developer\" Edition, BETA 0.2a-1",
                           WS_OVERLAPPEDWINDOW,
                           50, 50, 488,406,
                           HWND_DESKTOP,
                           NULL,
                           hThisInst,
                           NULL );

      // Store the Global Application Instance Handle

      hInst = hThisInst;

      // Initialise the Common Controls Library (Mainly for Open File Dialog)

      InitCommonControls();

      // Display and Update the Application Window

      ShowWindow( hwnd, nWinMode );
      UpdateWindow( hwnd );

      // Windows Message Processing Loop

      while( GetMessage( &msg, NULL, 0, 0 ) )
      {
         TranslateMessage( &msg );
         DispatchMessage( &msg );
      }

      // Exit the Application

      return( msg.wParam );
   }


////////////////////////////////////////////////////////////////////////////////
// Application Window Message Processing

   LRESULT CALLBACK WindowFunc( HWND hwnd,      // Handle to Application Window
                                UINT message,   // Windows Message
                                WPARAM wParam,  // Message Parameter
                                LPARAM lParam ) // Message Parameter
   {
      HMENU hMenu, hSubMenu;           // Handle to the Menu and Sub Menus

      switch( message )
      {
         // Do Application Initialisation 

         case WM_CREATE:

            hwndMain = hwnd;

            break;

         // Update the Application Window

         case WM_PAINT: 

            break;

         // Handle Application Defined Windows Messages

         case WM_COMMAND:

            switch( LOWORD( wParam ) )
            {
               // Load and Begin Execution of Selected Nintendo 64 ROM

               case IDM_FILE_OPEN:  

                  // Load and Prepare ROM Image

                  if( LoadROMImage( hwnd ) )
                  {
                     // Enable/Disable Appropriate Menu Options

                     hMenu = GetMenu( hwnd );
                     hSubMenu = GetSubMenu( hMenu, 0 );

                     EnableMenuItem( hSubMenu, IDM_FILE_OPEN,  
                                     MF_BYCOMMAND | MF_GRAYED  );
                     EnableMenuItem( hSubMenu, IDM_FILE_EXECUTE, 
                                     MF_BYCOMMAND | MF_ENABLED );
                     EnableMenuItem( hSubMenu, IDM_FILE_RESET, 
                                     MF_BYCOMMAND | MF_ENABLED );
                     EnableMenuItem( hSubMenu, IDM_FILE_PROPERTIES,    
                                     MF_BYCOMMAND | MF_ENABLED );
                  }

                  // Initialise Memory

                  if( !InitMem() )
                  {
                     // Reset Emulator if Memory Init Fails

                     FreeMem();       
                     SetWindowText( hwnd, "NSFE \"Developer\" Edition"
                                          ", BETA 0.2a-1" );

                     // Enable/Disable Appropriate Menu Options

                     hMenu = GetMenu( hwnd );
                     hSubMenu = GetSubMenu( hMenu, 0 );

                     EnableMenuItem( hSubMenu, IDM_FILE_OPEN,  
                                     MF_BYCOMMAND | MF_ENABLED  );
                     EnableMenuItem( hSubMenu, IDM_FILE_EXECUTE, 
                                     MF_BYCOMMAND | MF_GRAYED );
                     EnableMenuItem( hSubMenu, IDM_FILE_RESET, 
                                     MF_BYCOMMAND | MF_GRAYED );
                     EnableMenuItem( hSubMenu, IDM_FILE_PROPERTIES,    
                                     MF_BYCOMMAND | MF_GRAYED );
                  }

                  // Initialise Emulated Hardware

                  InitHardware();

                  // Enable Debugger Menu Option

                  hMenu = GetMenu( hwnd );
                  hSubMenu = GetSubMenu( hMenu, 2 );
                  EnableMenuItem( hSubMenu, IDM_DEBUG_ENTER,  
                                  MF_BYCOMMAND | MF_ENABLED  );

                  break;

               // Start Execution of the Emulator

               case IDM_FILE_EXECUTE:

                  // Start Emulation

                  hMainThread = CreateThread( NULL, 0,
                                              (LPTHREAD_START_ROUTINE)
                                              ExecNintendo64,
                                              (LPVOID)hwndMain,
                                              0,
                                              &n64Tid );

                  // SetThreadPriority( hMainThread, 
                  //                    THREAD_PRIORITY_TIME_CRITICAL );

                  // Enable/Disable Appropriate Menu Options

                  hMenu = GetMenu( hwnd );
                  hSubMenu = GetSubMenu( hMenu, 0 );

                  EnableMenuItem( hSubMenu, IDM_FILE_EXECUTE, 
                                  MF_BYCOMMAND | MF_GRAYED );
                  break;

               // Reset Emulator

               case IDM_FILE_RESET:

                  FreeMem();
                  SetWindowText( hwnd, "NSFE \"Developer\" Edition"
                                       ", BETA 0.2a-1" );

                  // Enable/Disable Appropriate Menu Options

                  hMenu = GetMenu( hwnd );
                  hSubMenu = GetSubMenu( hMenu, 0 );

                  EnableMenuItem( hSubMenu, IDM_FILE_OPEN,  
                                  MF_BYCOMMAND | MF_ENABLED  );
                  EnableMenuItem( hSubMenu, IDM_FILE_EXECUTE, 
                                  MF_BYCOMMAND | MF_GRAYED );
                  EnableMenuItem( hSubMenu, IDM_FILE_RESET, 
                                  MF_BYCOMMAND | MF_GRAYED );
                  EnableMenuItem( hSubMenu, IDM_FILE_PROPERTIES,    
                                  MF_BYCOMMAND | MF_GRAYED );

                  break;

               // Display the ROM Properties

               case IDM_FILE_PROPERTIES:  

                  // Display the ROM Properties

                  DisplayROMProperties();

                  break;

               // Exit the Emulator

               case IDM_FILE_EXIT:   

                  // Send Application Termination Message

				  SendMessage( hwnd, WM_DESTROY, 0, 0L );

                  break;
   
               // Set Video Options

               case IDM_OPTIONS_VIDEO:  

                  DisplayVideoProperties();

                  break;

               // Set Audio Options

               case IDM_OPTIONS_SOUND:  

                  DisplayAudioProperties();

                  break;

               // Set Controller Options

               case IDM_OPTIONS_CONTROLLERS:

                  DisplayControllerProperties();

                  break;

               // Enter Debugger

               case IDM_DEBUG_ENTER:

                  // Display the Debug Windows

                  RegisterDebugWindow();
                  MainDebugWindow();

                  // Set Appropriate Debug Menu Options

                  hMenu = GetMenu( hwnd );
                  hSubMenu = GetSubMenu( hMenu, 2 );
                  EnableMenuItem( hSubMenu, IDM_DEBUG_ENTER,  
                                  MF_BYCOMMAND | MF_GRAYED  );
                  EnableMenuItem( hSubMenu, IDM_DEBUG_EXIT,  
                                  MF_BYCOMMAND | MF_ENABLED  );

                  inDebugger = TRUE;

                  // Start Execution

                  SendMessage( hwnd, WM_COMMAND, IDM_FILE_EXECUTE, 0 ); 

                  break;

               // Exit Debugger

               case IDM_DEBUG_EXIT:

                  // Close the Debug Windows & Reset Emu ?

                  // Set Appropriate Debug Menu Options

                  hMenu = GetMenu( hwnd );
                  hSubMenu = GetSubMenu( hMenu, 2 );
                  EnableMenuItem( hSubMenu, IDM_DEBUG_ENTER,  
                                  MF_BYCOMMAND | MF_ENABLED  );
                  EnableMenuItem( hSubMenu, IDM_DEBUG_EXIT,  
                                  MF_BYCOMMAND | MF_GRAYED  );

                  inDebugger = FALSE;

                  break;

               // Display Application Help

               case IDM_HELP_HELPTOPICS:  

                  break;

               // Display Information About the Emulator

               case IDM_HELP_ABOUT:       

                  DialogBox( hInst, MAKEINTRESOURCE( IDD_ABOUT ),
                             hwnd, AboutDialog );
                  break;

               // Ignore any other Application Defined Messages

               default:                   

                  break;
            }

            break;

         // Process a Window Resize Action

         case WM_SIZE:                    

            // Invalidate the Client Area so a WM_PAINT Message is Sent

            InvalidateRect( hwnd, NULL, 1 );

            break;

         // Terminate the Application

         case WM_DESTROY: 

            // Do whatever tidying up is needed

            if( romLoaded )
            {
               // Clear Allocated Memory

               FreeMem();
            }

            PostQuitMessage( 0 );

            break;

         // Handle any Unhandle Windows Messages

         default:                         

            return( DefWindowProc( hwnd, message, wParam, lParam ) );

            break;
      }

      // Return from Main Window Message Handling Procedure

      return( 0L );
   }

   