////////////////////////////////////////////////////////////////////////////////
// Filename  : register.h
// File Desc : Internal registers
// Project   : NSFE (Nintendo Sixty Four Emulator)
// Release   : 0.2a (Developer Release)
// Copyright : NSFE is the Copyright (c) 1999, Brian Faber.  
//
// This software is a Windows implementation of TrueReality, Copyright (C) 1999,
// Niki W. Waibel, and is published under the terms of the GNU General Public 
// License, see below:
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and or modify it under
// the terms of the GNU General Public Licence as published by the Free Software
// Foundation; either version 2 of the Licence, or any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public Licence for more details
//
// You should have received a copy of the GNU General Public Licence along with 
// this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
// Ave, Cambridge, MA 02139, USA.
//
// NSFE Author Information:
//    Brian Faber, Chino, Califorina - USA
//    bevis590@gte.net
//
//    Ben Marsh, Townsville, Queensland - Australia
//    photi@ozemail.com.au
//
// Original TrueReality Author Information:
//    Niki W. Waibel, Reichenau 20, 6890 Lustenau, Austria - EUROPE
//    niki.waibel@xms.net
// -----------------------------------------------------------------------------
// Update History
//
// 00001 16-02-1998 "Initial Code Release.                                     "

#ifndef _REGISTER_H_
#define _REGISTER_H_


////////////////////////////////////////////////////////////////////////////////
// Include Files

#ifndef _STDSDK_H_
#include "stdsdk.h"                    // Standard Win32 API Includes, etc.
#endif // _STDSDK_H_

#ifndef _TYPEDEFS_H_
#include "typedefs.h"                  // Type Definitions
#endif // _TYPEDEFS_H_


////////////////////////////////////////////////////////////////////////////////
// Core R4300i Microprocessor - Registers

   // General Purpose Registers (GPRs)
      
   s64 gpReg[32];

   // Program Counter (PC)

   u32 PC;

   // HI Register - Integer Multiply/Divide High Order Result.

   u64 HI;

   // LO Register - Integer Multiply/Divide Low Order Result.

   u64 LO;

   // Load/Link LLBit Register (Only the first low order bit is used)

   u8 LLBit;


////////////////////////////////////////////////////////////////////////////////
// CP0 (System Control Co-processor) - Registers
// -----------------------------------------------------------------------------
// This co-processor supports a Virtual Memory System, selects Operating Mode
// and Controls Exceptions.

   //  0 Index    Programmable Pointer into TLB Array
   //  1 Random   Pseudorandom Pointer into TLB Array (Read Only)
   //  2 EntryLo0 Low Half of TLB Entry for Even Virtual Adress (VPN)
   //  3 EntryLo1 Low Half of TLB Entry for Odd Virtual Adress (VPN)
   //  4 Context  Pointer to Kernel Virtual Page Table Entry (PTE) in 32Bit Mode
   //  5 PageMask Page Size Specification
   //  6 Wired    Number of Wired TLB Entries
   //  7 -------- Reserved for Future Use
   //  8 BadVAddr Display of Virtual Address that Occured an Error Last
   //  9 Count    Timer Count
   // 10 EntryHi  High Half of TLB Entry (include ASID)
   // 11 Compare  Timer Compare Value
   // 12 Status   Operation Status Setting
   // 13 Cause    Display of Cause of Last Exception
   // 14 EPC      Exception Program Counter
   // 15 PRId     Processor Revision ID
   // 16 Config   Memory System Mode Setting
   // 17 LLAddr   Load Linked Instruction Address Display
   // 18 WatchLo  Memory Reference Trap Address Low Bits
   // 19 WatchHi  Memory Reference Trap Address High Bits
   // 20 XContext Pointer to Kernel Virtual PTE Table in 64-Bit Mode
   // 21 -------- Reserved for Future Use
   // 22 -------- Reserved for Future Use
   // 23 -------- Reserved for Future Use
   // 24 -------- Reserved for Future Use
   // 25 -------- Reserved for Future Use
   // 26 PErr     (Parity Error) Cache Parity Bits (R4200 Compatability Only)
   // 27 CErr     (Cache Error) Cache Error/Status Register (R4200 Compat. Only)
   // 28 TagLo    Cache Tag Register Low
   // 29 TagHi    Cache Tag Register High
   // 30 ErrorEPC Error Exception Program Counter
   // 31 -------- Reserved for Future Use

   u64 cp0Reg[32];


////////////////////////////////////////////////////////////////////////////////
// CP1 (Floating Point Co-processor) - Registers

   // Floating Point General Purpose Registers

   u64 cp1Reg[32];

   // Floating Point Implementation/Revision Register

   u32 FCR0;

   // Floating Point Control/Status Register

   u32 FCR31;


////////////////////////////////////////////////////////////////////////////////
// CP2 (Reality Co-processor) - Registers

   // General Purpose Registers
      
   s64 cp2Reg[32];

   // Vector Registers

   VECTOR vReg[32];


////////////////////////////////////////////////////////////////////////////////
// Globals

   TLB tlb[32];                        // Translation Lookaside Buffer


#endif // _REGISTER_H_
