////////////////////////////////////////////////////////////////////////////////
// Filename  : rom.c
// File Desc : ROM Image Handling
// Project   : NSFE (Nintendo Sixty Four Emulator)
// Release   : 0.2a (Developer Release)
// Copyright : NSFE is the Copyright (c) 1999, Brian Faber.  
//
// This software is a Windows implementation of TrueReality, Copyright (C) 1999,
// Niki W. Waibel, and is published under the terms of the GNU General Public 
// License, see below:
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and or modify it under
// the terms of the GNU General Public Licence as published by the Free Software
// Foundation; either version 2 of the Licence, or any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public Licence for more details
//
// You should have received a copy of the GNU General Public Licence along with 
// this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
// Ave, Cambridge, MA 02139, USA.
//
// NSFE Author Information:
//    Brian Faber, Chino, Califorina - USA
//    bevis590@gte.net
//
//    Ben Marsh, Townsville, Queensland - Australia
//    photi@ozemail.com.au
//
// Original TrueReality Author Information:
//    Niki W. Waibel, Reichenau 20, 6890 Lustenau, Austria - EUROPE
//    niki.waibel@xms.net
// -----------------------------------------------------------------------------
// Update History
//
// 00001 16-02-1998 "Initial Code Release.                                     "
// 00002 21-08-1998 "Optimised Assembler ROM Initialisation by Dan Guisinger.  "

#ifndef _ROM_H_
#include "rom.h"
#endif // _ROM_H_


////////////////////////////////////////////////////////////////////////////////
// Load the ROM Image

   BOOL LoadROMImage( HANDLE hwnd )
   {
      OPENFILENAME OpenFileName;       // Structure for Open Common Dialog
      TCHAR szFile[ MAX_PATH ] = "\0"; // Filename
      HANDLE hFile;                    // Handle to ROM Image File
      DWORD dwBytesRead;               // Number of Bytes Read
      DWORD i;                         // Loop/Index Iterator
      PVOID mem;                       // Temporary Memory Pointer

      // Open Filename Flags

      DWORD ofnFlags = OFN_FILEMUSTEXIST | 
                       OFN_HIDEREADONLY  |
                       OFN_PATHMUSTEXIST;

      // File Type Filter

      LPCTSTR szFilter = "Doctor V64 (*.v64, *.u64, *.n64)\0"  // Filter Desc 1
                         "*.v64;*.u64;*.n64\0"                 // Extentions 1
                         "Mr Backup (*.bin)\0"                 // Filter Desc 2
                         "*.bin\0"                             // Extentions 2
                         "All Files (*.*)\0"                   // Filter Desc 3
                         "*.*\0\0";                            // Extentions 3

      // Fill in the OPENFILENAME Structure

      OpenFileName.lStructSize       = sizeof( OPENFILENAME );
      OpenFileName.hwndOwner         = hwnd;
      OpenFileName.hInstance         = hInst;
      OpenFileName.lpstrFilter       = szFilter;
      OpenFileName.lpstrCustomFilter = (LPTSTR) NULL;
      OpenFileName.nMaxCustFilter    = 0;
      OpenFileName.nFilterIndex      = 1L;
      OpenFileName.lpstrFile         = szFile;
      OpenFileName.nMaxFile          = sizeof( szFile );
      OpenFileName.lpstrFileTitle    = NULL;
      OpenFileName.nMaxFileTitle     = 0;
      OpenFileName.lpstrInitialDir   = NULL;
      OpenFileName.lpstrTitle        = "Open Nintendo 64 Rom Image";
      OpenFileName.nFileOffset       = 0;
      OpenFileName.nFileExtension    = 0;
      OpenFileName.lpstrDefExt       = "*.v64";
      OpenFileName.Flags             = ofnFlags;
      OpenFileName.lCustData         = 0;
      OpenFileName.lpfnHook          = NULL;        
      OpenFileName.lpTemplateName    = NULL;

      // Call the Common Dialog Function

      if( GetOpenFileName( &OpenFileName ) )
      {
   		// Open the ROM Image File Selected

		   if( ( hFile = CreateFile( (LPCTSTR)OpenFileName.lpstrFile,
			                          GENERIC_READ,
			                          FILE_SHARE_READ,
			                          NULL,
			                          OPEN_EXISTING,
			                          FILE_ATTRIBUTE_NORMAL,
			                          (HANDLE)NULL ) ) == (HANDLE) - 1 )
         {
            // File Open Failed

            wsprintf( szBuffer, "File open failed (%S)",
                      OpenFileName.lpstrFile );
	    	   MessageBox( hwnd, szBuffer, "Application Error", MB_OK );

	    	   return( FALSE );
		   }

         // Get the Size of the Open ROM Image

		   dwFileSize = GetFileSize( hFile, NULL );
		   
         // Could not Obtain File Size

         if( dwFileSize == 0xFFFFFFFF )
		   {
	    	   MessageBox( NULL, "Could not determine ROM image size.", 
                              "Application Error", MB_OK );

            CloseHandle( hFile );
	    	   
            return( FALSE );
         }

         // Allocate Memory for the Selected Nintendo 64 ROM Image

		   romImage = (u32 *)GlobalAlloc(GMEM_FIXED | GMEM_ZEROINIT, dwFileSize);
		   
         if( romImage == NULL )
		   {
            MessageBox( NULL, "Could not allocate memory for ROM image.",
                              "Application Error", MB_OK );

	    	   CloseHandle( hFile );

	    	   return( FALSE );
         }

         // Read the ROM Image into Memory

         SetWindowText( hwnd, "NSFE \"Developer\" Edition, BETA 0.2a-1"
                              " - Loading ROM Image" );

         if( !ReadFile( hFile, (LPVOID) romImage, dwFileSize, 
                        &dwBytesRead, NULL ) )
         {
            MessageBox( NULL, "Could not read ROM image into memory",
                              "Application Error", MB_OK );

            GlobalFree( romImage );
	    	   CloseHandle( hFile );

	    	   return( FALSE );
         }

         // Prepare ROM Image if not in .BIN format
 
         SetWindowText( hwnd, "NSFE \"Developer\" Edition, BETA 0.2a-1"
                              " - Initialising ROM Image" );

         if( LOWORD( romImage[0] ) == 0x8037 )
         {
            // Word Swap Boot/Game Code
      
            for( i = 0; i < (dwFileSize / 4); i++ )
            {
               mem = &romImage[i];
               __asm {
                  mov   ebx,mem
                  ror   dword ptr SS:[ebx],16
               }
            }
         }
         else
         {
            // Word and Byte Swap Boot/Game Code
      
            for( i = 0; i < (dwFileSize / 4); i++ )
            {
               mem = &romImage[i];
               __asm {
                  mov   ebx,mem
                  mov   eax,dword ptr SS:[ebx]
                  xchg  al,ah
                  ror   eax,16
                  xchg  al,ah
                  mov   dword ptr SS:[ebx],eax
               }
            }
         }

         // Initialise ROM Header Structure

         ROM = (pRomImageHeader)romImage;

         // Display the ROM Image Name on the Title Bar

         for( i = 0; i < 20; i += 4)
         {
            cartridgeName[i+0] = ROM->name[i+3];
            cartridgeName[i+1] = ROM->name[i+2];
            cartridgeName[i+2] = ROM->name[i+1];
            cartridgeName[i+3] = ROM->name[i+0];
         }

         cartridgeName[20] = '\0';

         wsprintf( szBuffer, "NSFE \"Developer\" Edition, BETA 0.2a-1 - [%hS]",
                   cartridgeName );
         SetWindowText( hwnd, szBuffer );

         // Set Cartridge Size

         cartridgeSize = (dwFileSize - 0x1000);

         // Set Boot Address

         baseAddress = 0xA4000040;

         // Close File as ROM Image is now in Memory

         romLoaded = TRUE;
         CloseHandle( hFile );
      }
      else
      {
         // GetOpenFileName() Has Failed

         return( FALSE );
      }

      // ROM is Now Ready to Execute

      return( TRUE );
   }


////////////////////////////////////////////////////////////////////////////////
// Define the Property Sheet Page for the ROM Properties

   int DisplayROMProperties( void )
   {
      PROPSHEETPAGE   psp[1];
      PROPSHEETHEADER psh;

      // Define the Property Sheet Page

      psp[0].dwSize      = sizeof( PROPSHEETPAGE );
      psp[0].dwFlags     = PSP_USETITLE;
      psp[0].hInstance   = hInst;
      psp[0].pszTemplate = MAKEINTRESOURCE( IDD_ROMPROPERTIES );
      psp[0].pszIcon     = NULL;
      psp[0].pfnDlgProc  = (DLGPROC)RomProperties;
      psp[0].pszTitle    = "ROM Cartridge";
      psp[0].lParam      = 0;

      // Define Property Sheet Header

      psh.dwSize     = sizeof( PROPSHEETHEADER );
      psh.dwFlags    = PSH_PROPSHEETPAGE | PSH_NOAPPLYNOW | PSH_USEICONID;
      psh.hwndParent = hwndMain;
      psh.hInstance  = hInst;
      psh.pszIcon    = MAKEINTRESOURCE( IDI_N64CART );
      psh.pszCaption = (LPSTR)"Properties";
      psh.nPages     = 1;
      psh.nStartPage = 0;
      psh.ppsp       = (LPCPROPSHEETPAGE) &psp;

      return( PropertySheet( &psh ) );
   }


////////////////////////////////////////////////////////////////////////////////
// ROM Properties Tabbed Dialog

   BOOL APIENTRY RomProperties( HWND hDlg, UINT message, 
                                UINT wParam, LONG lParam )
   {
      static PROPSHEETPAGE *ps;        // Pointer to Property Sheet Information

      switch( message )
      {
         case WM_INITDIALOG:

            // Save the PROPSHEETPAGE Information

            ps = (PROPSHEETPAGE *)lParam;
            return( TRUE );
            break;

         case WM_NOTIFY:

            switch( ((NMHDR FAR *)lParam)->code )
            {
               case PSN_SETACTIVE:

                  // Initialise the Information Fields

                  SetDlgItemText( hDlg, IDC_ROMNAME, cartridgeName );

                  switch( ROM->manufacturerID )
                  {
                     case 0x0000004E:
                        wsprintf( szBuffer, "Nintendo" );
                        break;
                     default:
                        wsprintf( szBuffer, "Unknown Identifier (%Xh)",
                                  ROM->manufacturerID );
                        break;
                  }
                  SetDlgItemText( hDlg, IDC_MANUFACTURER, szBuffer );

                  switch( GETCOUNTRYCODE )
                  {
                     case 0x44: 
                        wsprintf( szBuffer, "Germany" );
                        break;
                     case 0x45:
                        wsprintf( szBuffer, "United States" );
                        break;
                     case 0x4A:
                        wsprintf( szBuffer, "Japan" );
                        break;
                     case 0x50:
                        wsprintf( szBuffer, "Europe" );
                        break;
                     case 0x55:
                        wsprintf( szBuffer, "Australia" );
                        break;
                     default:
                        wsprintf( szBuffer, "Unknown Country Code (%Xh)",
                                  GETCOUNTRYCODE );
                        break;
                  }
                  SetDlgItemText( hDlg, IDC_COUNTRY, szBuffer );

                  switch( cartridgeSize )
                  {
                     case ROM8MBit:
                        wsprintf( szBuffer, "8MBit" );
                        break;
                     case ROM16MBit:
                        wsprintf( szBuffer, "16MBit" );
                        break;
                     case ROM32MBit:
                        wsprintf( szBuffer, "32MBit" );
                        break;
                     case ROM64MBit:
                        wsprintf( szBuffer, "64MBit" );
                        break;
                     case ROM96MBit:
                        wsprintf( szBuffer, "96MBit" );
                        break;
                     case ROM128MBit:
                        wsprintf( szBuffer, "128MBit" );
                        break;
                     case ROM256MBit:
                        wsprintf( szBuffer, "256MBit" );
                        break;
                     case ROM512MBit:
                        wsprintf( szBuffer, "512MBit" );
                        break;
                     default:
                        wsprintf( szBuffer, "Non-Standard" );
                        break;
                  }
                  SetDlgItemText( hDlg, IDC_ROMSIZE, szBuffer );

                  wsprintf( szBuffer, "%c%c", 
                            (char)(GETCARTRIDGEID >> 8) & 0x00FF,
                            (char)(GETCARTRIDGEID & 0x00FF) ); 
                  SetDlgItemText( hDlg, IDC_CARTRIDGEID, szBuffer );

                  wsprintf( szBuffer, "0x%08X", baseAddress );
                  SetDlgItemText( hDlg, IDC_BASEADDR, szBuffer );

                  wsprintf( szBuffer, "0x%08X", ROM->gameAddress );
                  SetDlgItemText( hDlg, IDC_GAMEADDR, szBuffer );

                  wsprintf( szBuffer, "0x%08X", ROM->CRC1 );
                  SetDlgItemText( hDlg, IDC_CRC1, szBuffer );

                  wsprintf( szBuffer, "0x%08X", ROM->CRC2 );
                  SetDlgItemText( hDlg, IDC_CRC2, szBuffer );

                  wsprintf( szBuffer, "0x%08X", ROM->release );
                  SetDlgItemText( hDlg, IDC_RELEASE, szBuffer );

                  wsprintf( szBuffer, "0x%08X", ROM->clockRate );
                  SetDlgItemText( hDlg, IDC_CLOCKRATE, szBuffer );

                  break;

               case PSN_APPLY:

                  SetWindowLong( hDlg, DWL_MSGRESULT, TRUE );
                  break;

               case PSN_KILLACTIVE:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );
                  return( 1 );
                  break;

               case PSN_RESET:

                  SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );
                  break;

               default:

                  break;
            }

            break;
      }

      return( FALSE );
   }