////////////////////////////////////////////////////////////////////////////////
// Filename  : rom.h
// File Desc : ROM Image Handling
// Project   : NSFE (Nintendo Sixty Four Emulator)
// Release   : 0.2a (Developer Release)
// Copyright : NSFE is the Copyright (c) 1999, Brian Faber.  
//
// This software is a Windows implementation of TrueReality, Copyright (C) 1999,
// Niki W. Waibel, and is published under the terms of the GNU General Public 
// License, see below:
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and or modify it under
// the terms of the GNU General Public Licence as published by the Free Software
// Foundation; either version 2 of the Licence, or any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public Licence for more details
//
// You should have received a copy of the GNU General Public Licence along with 
// this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
// Ave, Cambridge, MA 02139, USA.
//
// NSFE Author Information:
//    Brian Faber, Chino, Califorina - USA
//    bevis590@gte.net
//
//    Ben Marsh, Townsville, Queensland - Australia
//    photi@ozemail.com.au
//
// Original TrueReality Author Information:
//    Niki W. Waibel, Reichenau 20, 6890 Lustenau, Austria - EUROPE
//    niki.waibel@xms.net
// -----------------------------------------------------------------------------
// Update History
//
// 00001 16-02-1998 "Initial Code Release.                                     "
// 00002 21-08-1998 "Optimised Assembler ROM Initialisation by Dan Guisinger.  "

#ifndef _ROM_H_
#define _ROM_H_


////////////////////////////////////////////////////////////////////////////////
// Include Files

#ifndef _STDSDK_H_
#include "stdsdk.h"                    // Standard Win32 API Includes, etc.
#endif // _STDSDK_H_

#ifndef _TYPEDEFS_H_
#include "typedefs.h"                  // Type Definitions
#endif // _TYPEDEFS_H_


////////////////////////////////////////////////////////////////////////////////
// Procedure Prototypes

   BOOL LoadROMImage( HANDLE );
   int DisplayROMProperties( void );
   BOOL APIENTRY RomProperties( HWND, UINT, UINT, LONG );


////////////////////////////////////////////////////////////////////////////////
// Macros

#define  GETCARTRIDGEID    (WORD)((ROM->cartridgeInfo) >> 16 & 0x0000FFFF)
#define  GETCOUNTRYCODE    (BYTE)((ROM->cartridgeInfo) >> 8 & 0x000000FF)

#define  ROM4MBit            524288    // 4MBit ROM Image Size
#define  ROM8MBit           1048576    // 8MBit ROM Image Size
#define  ROM16MBit          2097152    // 16MBit ROM Image Size
#define  ROM32MBit          4194304    // 32MBit ROM Image Size
#define  ROM64MBit          8388608    // 64MBit ROM Image Size
#define  ROM96MBit         12582912    // 96MBit ROM Image Size
#define  ROM128MBit        16777216    // 128MBit ROM Image Size
#define  ROM256MBit        33554432    // 256MBit ROM Image Size
#define  ROM512MBit        67108864    // 512MBit ROM Image Size


////////////////////////////////////////////////////////////////////////////////
// Globals

   BOOL romLoaded = FALSE;             // Has the ROM been Loaded ?
   DWORD dwFileSize;                   // Returned ROM Size
   pRomImageHeader ROM;                // Rom Header Information
   u8 cartridgeName[21];               // ROM Cartridge Name
   u32 cartridgeSize;                  // ROM Cartridge Size (in Bytes)
   u32 baseAddress;                    // Boot Code Base Address

   extern HINSTANCE hInst;             // Global Application Instance
   extern char szBuffer[ MAX_PATH ];   // Temporary Storage
   extern u32 *romImage;               // Header, Boot and Game Code
   extern HWND hwndMain;               // Handle to the Main App Window


#endif // _ROM_H_
