////////////////////////////////////////////////////////////////////////////////
// Filename  : typedefs.h
// File Desc : Type Definitions
// Project   : NSFE (Nintendo Sixty Four Emulator)
// Release   : 0.2a (Developer Release)
// Copyright : NSFE is the Copyright (c) 1999, Brian Faber.  
//
// This software is a Windows implementation of TrueReality, Copyright (C) 1999,
// Niki W. Waibel, and is published under the terms of the GNU General Public 
// License, see below:
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and or modify it under
// the terms of the GNU General Public Licence as published by the Free Software
// Foundation; either version 2 of the Licence, or any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public Licence for more details
//
// You should have received a copy of the GNU General Public Licence along with 
// this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
// Ave, Cambridge, MA 02139, USA.
//
// NSFE Author Information:
//    Brian Faber, Chino, Califorina - USA
//    bevis590@gte.net
//
//    Ben Marsh, Townsville, Queensland - Australia
//    photi@ozemail.com.au
//
// Original TrueReality Author Information:
//    Niki W. Waibel, Reichenau 20, 6890 Lustenau, Austria - EUROPE
//    niki.waibel@xms.net
// -----------------------------------------------------------------------------
// Update History
//
// 00001 16-02-1998 "Initial Code Release.                                     "

#ifndef _TYPEDEFS_H_
#define _TYPEDEFS_H_


////////////////////////////////////////////////////////////////////////////////
// General Data Types
 
typedef unsigned char         u8;      // Unsigned  8-Bit
typedef unsigned __int16      u16;     // Unsigned 16-Bit
typedef unsigned __int32      u32;     // Unsigned 32-Bit
typedef unsigned __int64      u64;     // Unsigned 64-Bit

typedef signed __int8         s8;      // Signed  8-Bit
typedef signed __int16        s16;     // Signed 16-Bit
typedef signed __int32        s32;     // Signed 32-Bit
typedef signed __int64        s64;     // Signed 64-Bit

typedef float                 f32;     // Single Precision Floating Point
typedef double                f64;     // Double Precision Floating Point


////////////////////////////////////////////////////////////////////////////////
// Rom Header Information

   typedef struct _RomImageHeader
   {
      u32 notUsed00;                   // 00
      u32 clockRate;                   // 01 - Clock Rate
      u32 gameAddress;                 // 02 - Base Game Address
      u32 release;                     // 03 - Release
      u32 CRC1;                        // 04 - CRC Checksum 1
      u32 CRC2;                        // 05 - CRC Checksum 2
      u32 notUsed06;                   // 06
      u32 notUsed07;                   // 07
      u8  name[20];                    // 08, 09, 0A, 0B, 0C - Game Name
      u32 notUsed0D;                   // 0D
      u32 manufacturerID;              // 0E - Manufacturer Identification
      u32 cartridgeInfo;               // 0F - Cartridge Information
   } RomImageHeader, *pRomImageHeader;


////////////////////////////////////////////////////////////////////////////////
// Interrupts

   typedef struct _Interrupt
   {
      u64 intrTime;                    // Interrupt Raise Time
      void *intrHandler;               // Interrupt Handler
   } INTERRUPTS;


////////////////////////////////////////////////////////////////////////////////
// TLB (Translation Lookaside Buffer)

   typedef struct _Tlb
   {
      u64 PageMask;
      u64 EntryHi;
      u64 EntryLo1;
      u64 EntryLo0;
      u64 G;
   } TLB;


////////////////////////////////////////////////////////////////////////////////
// Vector Register Structure

   typedef struct _Vector
   {
      s16 e7;                          // Vector Element 7
      s16 e6;                          // Vector Element 6
      s16 e5;                          // Vector Element 5
      s16 e4;                          // Vector Element 4
      s16 e3;                          // Vector Element 3
      s16 e2;                          // Vector Element 2
      s16 e1;                          // Vector Element 1
      s16 e0;                          // Vector Element 0
   } VECTOR;


#endif // _TYPEDEFS_H_

